IMPLEMENTATION MODULE OCompute;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


(********************** IMPORT from system libraries *)
FROM SYSTEM             IMPORT ADDRESS;
FROM Terminal           IMPORT BusyRead;
FROM InOut              IMPORT Read;

FROM GEMAESbase         IMPORT FormStart,
                               FormGrow,
                               FormShrink,
                               FormFinish,
                               Normal,
                               Object;
FROM AESForms           IMPORT FormAlert,
                               FormDo,
                               FormDialogue;
FROM AESObjects         IMPORT ObjectDraw,
                               ObjectChange;

(********************** IMPORT from ORBIT specific libraries *)
FROM OGEMGlobal         IMPORT WorkstationHandle,
                               HaltedTree,
                               Touched;
FROM OrbGlobal          IMPORT StartYear,
                               StartMonth,
                               StartDay,
                               StartHours,
                               StartMinutes,
                               DurationHours,
                               DurationMinutes,
                               TimeStep,
                               DopplerOK,
                               OmloopNr;
FROM OOutput            IMPORT PrintOverzicht,
                               PrintHeading,
                               PrintSlotRegel,
                               PrintNewDay,
                               PrintNewOrbit,
                               PrintDataRegel;
FROM OSubs              IMPORT EigenGeocentrisch,
                               SiderTijd1Jan,
                               RelStartTijd,
                               InitOrbElements,
                               BepaalRotMat,
                               BepaalAnomalieOmloop,
                               SolveKeppler,
                               SolveParams,
                               SolveDoppler;
FROM OClock             IMPORT TimeRecord,
                               ReadTime;
FROM OFiler             IMPORT Station,
                               SatRec;



VAR T,
    TStart,
    TEind,
    StepInSec:          REAL;
    Day,
    DayNr,
    OrbNr:              CARDINAL;
    Ch:                 CHAR;
    TR:                 TimeRecord;



PROCEDURE DoHalt;
VAR Xf,Yf,Wf,Hf: INTEGER;
    ObjPtr:      POINTER TO ARRAY[0..1] OF Object;
BEGIN
  Ch:=0C;
  ObjPtr:=ADDRESS(HaltedTree);
  ObjPtr^[0].x:=400;
  ObjPtr^[0].y:=21;
  Xf:=ObjPtr^[0].x;
  Yf:=ObjPtr^[0].y;
  Wf:=ObjPtr^[0].width;
  Hf:=ObjPtr^[0].height;
  FormDialogue(FormStart,Xf+(Wf DIV 2),Yf+(Hf DIV 2),1,1,Xf,Yf,Wf,Hf);
  FormDialogue(FormGrow,Xf+(Wf DIV 2),Yf+(Hf DIV 2),1,1,Xf,Yf,Wf,Hf);
  ObjectDraw(HaltedTree,0,2,Xf,Yf,Wf,Hf);
  Touched:=FormDo(HaltedTree,-1);
  FormDialogue(FormShrink,Xf+(Wf DIV 2),Yf+(Hf DIV 2),1,1,Xf,Yf,Wf,Hf);
  FormDialogue(FormFinish,Xf+(Wf DIV 2),Yf+(Hf DIV 2),1,1,Xf,Yf,Wf,Hf);
  ObjectChange(HaltedTree,Touched,0,Xf,Yf,Wf,Hf,Normal,0)
END DoHalt;



PROCEDURE ReadGMT():REAL;
VAR Ret: REAL;
BEGIN
  ReadTime(TR);
  IF Station.EigenTimeGMT<0 THEN
    Ret:=((RelStartTijd(TR.Year,TR.Month,TR.Day,TR.Hour,TR.Min))
        +(FLOAT(TR.Sec)/86400.0)
        +(FLOAT(CARDINAL(ABS(Station.EigenTimeGMT)))/24.0))
  ELSE
    Ret:=((RelStartTijd(TR.Year,TR.Month,TR.Day,TR.Hour,TR.Min))
        +(FLOAT(TR.Sec)/86400.0)
        -(FLOAT(CARDINAL(Station.EigenTimeGMT))/24.0))
  END;
  RETURN Ret
END ReadGMT;


PROCEDURE Kern;
BEGIN
  Day:=TRUNC(T);
  IF DayNr # Day THEN
    DayNr:=TRUNC(T);
    PrintNewDay(DayNr)
  END;
  BepaalAnomalieOmloop(T);
  IF OrbNr # OmloopNr THEN
    OrbNr:=OmloopNr;
    PrintNewOrbit(OrbNr);
    BepaalRotMat(T)
  END;
  SolveKeppler(T);
  SolveParams(T,StepInSec);
  SolveDoppler;
  PrintDataRegel(T)
END Kern;



PROCEDURE Scan;
VAR Dum: INTEGER;
BEGIN
  EigenGeocentrisch;
  SiderTijd1Jan;
  IF TimeStep>1.0 THEN
    DopplerOK:=TRUE
  ELSE
    DopplerOK:=FALSE
  END;
  TStart:=RelStartTijd(StartYear,StartMonth,StartDay,
                       StartHours,StartMinutes);
  IF TStart<=SatRec.YDag THEN
    Dum:=FormAlert(1,
           '[1][ |The starttime must lie after|the reference epoch!][Ok]')
  ELSE
    TEind:=TStart+FLOAT(DurationHours)/24.0+FLOAT(DurationMinutes)/1440.0;
    InitOrbElements;
    BepaalRotMat(TStart);
    BepaalAnomalieOmloop(TStart);
    PrintOverzicht(TStart,TEind);
    PrintHeading;
    DayNr:=65000;
    OrbNr:=65000;
    StepInSec:=TimeStep*60.0;
    T:=TStart;
    WHILE T<=TEind DO
      Kern;
      T:=T+(TimeStep/1440.0);
      BusyRead(Ch);
      IF ((Ch='h') OR (Ch='H')) THEN
        DoHalt
      END;
      IF Ch#0C THEN
        T:=TEind+1.0
      END
    END;
    PrintSlotRegel
  END
END Scan;


PROCEDURE Track;
VAR Toud: REAL;
    Dum: INTEGER;
BEGIN
  DopplerOK:=FALSE;
  EigenGeocentrisch;
  SiderTijd1Jan;
  T:=ReadGMT();
  IF T<=SatRec.YDag THEN
    Dum:=FormAlert(1,'[1][System time must lie after|the reference epoch!| |System clock not set?][Ok]')
  ELSE
    InitOrbElements;
    BepaalRotMat(T);
    BepaalAnomalieOmloop(T);
    PrintHeading;
    DayNr:=65000;
    OrbNr:=65000;
    StepInSec:=1.0;
    REPEAT
      Kern;
      Toud:=T;
      REPEAT
        T:=ReadGMT();
      UNTIL T#Toud;
      BusyRead(Ch);
      IF ((Ch='h') OR (Ch='H')) THEN
        DoHalt
      END
    UNTIL Ch#0C;
    PrintSlotRegel
  END
END Track;


END OCompute.
