IMPLEMENTATION MODULE OOutput;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


(***************** IMPORT from standard libraries *)
FROM SYSTEM        IMPORT ADR;
FROM GEMDOS        IMPORT Delete;
FROM ASCII         IMPORT ESC;
FROM Streams       IMPORT OpenStream,
                          CloseStream,
                          StreamKinds,
                          Stream;
FROM InOut         IMPORT Done,
                          OpenInputFile,
                          CloseInput;
FROM TextIO        IMPORT WRite,
                          WRiteString,
                          WRiteCard,
                          WRiteLn,
                          WRiteReal,
                          WriteLn,
                          WriteString,
                          Read;
FROM Strings       IMPORT String,
                          Assign,
                          Concat,
                          Copy,
                          Pos,
                          Length;
FROM ASCII         IMPORT NUL;
FROM M2Conversions IMPORT ConvertCardinal,
                          ConvertReal,
                          ConvertInteger;
FROM MathLib0      IMPORT entier;

(***************** IMPORT from 'Orbit' specific libraries *)
FROM OrbGlobal     IMPORT M1e,
                          AardStraal,
                          RadGrad,
                          OmloopNr,
                          Anomalie,
                          Fase,
                          ArgPer,
                          Raan,
                          HalveAs,
                          Azimuth,
                          Elevation,
                          Distance,
                          DFreq,
                          HeightC,
                          WLongitude,
                          Latitude,
                          DopplerOK,
                          MonthNames,
                          ShowLowElevation,
                          ShowLittleChanges,
                          ElevationThres,
                          LittleChangesThres;
FROM OFiler        IMPORT SatRec,
                          Station;
FROM OGEMGlobal    IMPORT LoggingOn,
                          OutOnScreen,
                          GraphicsOut,
                          HandleForm,
                          Touched,
                          LogTreeAddr,
                          OVERWRIT,
                          NOLOG;
FROM OOutWindow    IMPORT OpenOutWindow,
                          WriteOutLine;

(****************) IMPORT GEMAESbase;
FROM AESForms      IMPORT FormAlert,
                          FileSelectorInput;



VAR  OutText:    Stream;
     Dummy,
     FileS:      INTEGER;
     OutString:  String;


PROCEDURE OutToPRN;
BEGIN
  OpenStream(OutText,'prn:',READWRITE,FileS);
  IF FileS#0 THEN
    Dummy:=FormAlert(1,'[3][ |Error opening PRN:][Abort]');
    HALT
  END
END OutToPRN;


PROCEDURE CloseOutToPRN;
BEGIN
  CloseStream(OutText,FileS);
  IF FileS#0 THEN
    Dummy:=FormAlert(1,'[3][ |Error closing PRN:][Abort]');
    HALT
  END
END CloseOutToPRN;


PROCEDURE MakeFileName(VAR Dir,Sel,Tot: String);
VAR LastValid,Start,Position: CARDINAL;
    Bool: BOOLEAN;
    Hulp: String;
BEGIN
  Start:=0;
  Position:=Length(Dir)-1;
  Hulp:='\';
  REPEAT  
    LastValid:=Position;
    Bool:=Pos(Dir,Hulp,Start,Position);
    Start:=Position+1
  UNTIL Bool=FALSE;
  Copy(Dir,0,LastValid+1,Hulp);
  Concat(Hulp,Sel,Tot)
END MakeFileName;



PROCEDURE OutToFile(VAR Ok:BOOLEAN);
VAR SelName,
    PathName,
    TotalName: String;
    Bool,
    DoIt:      BOOLEAN;
BEGIN
  Assign(PathName,'ORBIT.DAT\*.OUT');
  Assign(SelName,'ORBIT.OUT');
  FileSelectorInput(ADR(PathName),ADR(SelName),Touched);
  IF Touched=1 THEN
    MakeFileName(PathName,SelName,TotalName);
    DoIt:=TRUE;
    OpenInputFile(TotalName);
    IF Done THEN
      CloseInput;
      HandleForm(LogTreeAddr,-1);
      IF Touched=NOLOG THEN
        DoIt:=FALSE;
        Ok:=FALSE
      ELSE
        Bool:=Delete(TotalName);
      END
    END;
    IF DoIt THEN 
      OpenStream(OutText,TotalName,READWRITE,FileS);
      IF FileS#0 THEN
        Dummy:=FormAlert(1,'[3][ |Error creating file!][Abort]');
        HALT
      END;
      Ok:=TRUE
    END
  ELSE
    Ok:=FALSE
  END
END OutToFile;


PROCEDURE CloseOutToFile;
BEGIN
  CloseStream(OutText,FileS);
  IF FileS#0 THEN
    Dummy:=FormAlert(1,'[3][ |Error closing file.][Abort]');
    HALT
  END
END CloseOutToFile;



PROCEDURE MakeTimeString(Time: REAL; VAR TimeString:ARRAY OF CHAR);
  (*
     Maakt string 'hhmm:ss'
  *)
VAR Lengte, Sec, Min, Hrs, ZesHrs: CARDINAL;
    SecStr, MinStr, HrsStr: ARRAY[0..2] OF CHAR;
    SecReal, FracTime: REAL;
    Bool: BOOLEAN;
    TeGroot: ARRAY[0..7] OF CHAR;
BEGIN
  IF Time > 32767.0 THEN
    TeGroot:='<xxxxx>';
    Assign(TeGroot,TimeString)
  ELSE
    FracTime:=Time-FLOAT(TRUNC(Time));
    SecReal:=FracTime*86400.0;
    ZesHrs:=0;
    WHILE SecReal >= 21600.0 DO
      SecReal:=SecReal-21600.0;
      ZesHrs:=ZesHrs+6;
    END;
    Sec:=TRUNC(SecReal+0.5);
    Hrs:=Sec DIV 3600;
    Sec:=Sec-Hrs*3600;
    Hrs:=Hrs+ZesHrs;
    Min:=Sec DIV 60;
    Sec:=Sec-60*Min;
    ConvertCardinal(Hrs,1,HrsStr);
    ConvertCardinal(Min,1,MinStr);
    ConvertCardinal(Sec,1,SecStr);
    IF Length(HrsStr)=1 THEN
      HrsStr[1]:=HrsStr[0];
      HrsStr[0]:='0';
      HrsStr[2]:=NUL
    END;
    IF Length(MinStr)=1 THEN
      MinStr[1]:=MinStr[0];
      MinStr[0]:='0';
      MinStr[2]:=NUL
    END;
    IF Length(SecStr)=1 THEN
      SecStr[1]:=SecStr[0];
      SecStr[0]:='0';
      SecStr[2]:=NUL
    END;
    Concat(HrsStr,MinStr,TimeString);
    Lengte:=Length(TimeString);
    TimeString[Lengte]:=':';
    TimeString[Lengte+1]:=NUL;
    Concat(TimeString,SecStr,TimeString)
  END
END MakeTimeString;



PROCEDURE MakeDateString(Day:CARDINAL;VAR DateString:ARRAY OF CHAR);
  (*
    Maakt een dag-maand-jaar string uit een dagnummer.
    Format: 'dd-Mmm-yy'.
    Day=1, geeft 1-Jan-YY.
  *)
VAR YearString: ARRAY[0..2] OF CHAR;
    Lengte, HulpJaar, Maand, Correctie: CARDINAL;
    Bool: BOOLEAN;
  
  PROCEDURE SchrikkelDag(Jaar:CARDINAL):CARDINAL;
  (* Geeft 1 als Jaar een schrikkeljaar is, anders 0 *)
  VAR Schrik:CARDINAL;
  BEGIN
    IF (Jaar MOD 4)=0 THEN
      Schrik:=1
    ELSE
      Schrik:=0
    END;
    RETURN Schrik
  END SchrikkelDag;

BEGIN (* MakeDateString *)
  DEC(Day,1);       (* Day=1 => 1-Jan i.p.v. Day=0 => 1-Jan *)
  HulpJaar:=SatRec.YJaar;
  WHILE Day>=365+SchrikkelDag(HulpJaar) DO
    DEC(Day,365+SchrikkelDag(HulpJaar));
    INC(HulpJaar,1)
  END;
  Correctie:=SchrikkelDag(HulpJaar);
  Maand:=12;
  WHILE Day<M1e[Maand]+Correctie DO
    DEC(Maand,1);
    IF Maand=2 THEN
      Correctie:=0
    END
  END;
  Day:=Day-M1e[Maand]-Correctie+1;
  ConvertCardinal(Day,1,DateString);
  IF Length(DateString)=1 THEN
    DateString[1]:=DateString[0];
    DateString[0]:=' ';
    DateString[2]:=NUL
  END;
  ConvertCardinal(HulpJaar,1,YearString);
  IF Length(YearString)=1 THEN
    YearString[1]:=YearString[0];
    YearString[0]:='0';
    YearString[2]:=NUL
  END;
  Lengte:=Length(DateString);
  DateString[Lengte]:='-';
  DateString[Lengte+1]:=NUL;
  Concat(DateString,MonthNames[Maand],DateString);
  Lengte:=Length(DateString);
  DateString[Lengte]:='-';
  DateString[Lengte+1]:=NUL;
  Concat(DateString,YearString,DateString)
END MakeDateString;



PROCEDURE PrintOverzichtPrinter(StartTijd,Eindtijd:REAL);

  PROCEDURE PrintNumDayTime(Day:REAL);
  VAR Str: ARRAY[0..9] OF CHAR;
  BEGIN
    WRiteReal(OutText,Day,9,5);
    WRiteString(OutText,' = ');
    MakeDateString(TRUNC(Day),Str);
    WRiteString(OutText,Str);
    WRiteString(OutText,'  ');
    MakeTimeString(Day,Str);
    WRiteString(OutText,Str);
    WRiteLn(OutText)
  END PrintNumDayTime;

  PROCEDURE PrOverzicht1();
  BEGIN
    WRiteString(OutText,'Orbital elements for: ');
    WRiteString(OutText,SatRec.Naam);
    WRiteLn(OutText);
    WRiteString(OutText,'Reference epoch: ');
    WRiteCard(OutText,SatRec.YJaar,2);
    WRite(OutText,'+');
    PrintNumDayTime(SatRec.YDag);
    WRiteLn(OutText);
    WRiteString(OutText,'Station: ');
    WRiteString(OutText,Station.EigenNaam);
    WRiteString(OutText,'  (');
    WRiteReal(OutText,Station.EigenWest,7,3);
    WRiteString(OutText,' West, ');
    WRiteReal(OutText,Station.EigenNoord,6,3);
    WRiteString(OutText,' North, Height=');
    WRiteReal(OutText,Station.EigenHoogte,3,0);
    WRiteString(OutText,' m)');
    WRiteLn(OutText);
    WRiteLn(OutText);
    WRiteString(OutText,'From: ');
    PrintNumDayTime(StartTijd);
    WRiteString(OutText,'To:   ');
    PrintNumDayTime(Eindtijd);
    WRiteLn(OutText);
    WRiteString(OutText,
                 '------------------------------------------------------');
    WRiteLn(OutText);
    WRiteString(OutText,
                 '|  PARAMETER         |    REFERENCE  |     STARTING  |');
    WRiteLn(OutText);
    WRiteString(OutText,
                 '---------------------+---------------+----------------');
    WRiteLn(OutText)
  END PrOverzicht1;

  PROCEDURE PrOverzicht2();
  BEGIN
    WRiteString(OutText,'|  Orbit Number      |  ');
    WRiteCard(OutText,SatRec.YOmloop,11);
    WRiteString(OutText,'  |  ');
    WRiteCard(OutText,OmloopNr,11);
    WRiteString(OutText,'  |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Mean Anomaly      |    ');
    WRiteReal(OutText,SatRec.YMeanAnom,9,4);
    WRiteString(OutText,'  |    ');
    WRiteReal(OutText,Anomalie/RadGrad,9,4);
    WRiteString(OutText,'  |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Inclination       |     ');
    WRiteReal(OutText,SatRec.YInclinatie,8,4);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Eccentricity      |     ');
    WRiteReal(OutText,SatRec.YEccentr,8,6);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Mean Motion       |     ');
    WRiteReal(OutText,SatRec.YMeanMotion,8,4);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Acc. Mean Motion  |     ');
    WRiteReal(OutText,SatRec.YAccMeanM,8,-2);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Semi Major Axis   |  ');
    WRiteReal(OutText,HalveAs,11,4);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Arg. Perigee      |    ');
    WRiteReal(OutText,SatRec.YArgPer,9,4);
    WRiteString(OutText,'  |    ');
    WRiteReal(OutText,ArgPer,9,4);
    WRiteString(OutText,'  |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  R.A.A.N.          |    ');
    WRiteReal(OutText,SatRec.YRaan,9,4);
    WRiteString(OutText,'  |    ');
    WRiteReal(OutText,Raan,9,4);
    WRiteString(OutText,'  |');
    WRiteLn(OutText);
    WRiteString(OutText,'|  Frequency         |   ');
    WRiteReal(OutText,SatRec.Frekwentie,10,4);
    WRiteString(OutText,'  |               |');
    WRiteLn(OutText);
    WRiteString(OutText,
                 '------------------------------------------------------')
  END PrOverzicht2;

BEGIN (* PrintOverzichtPrinter *)
  PrOverzicht1;
  PrOverzicht2;
  WRiteLn(OutText);
  WRiteLn(OutText)
END PrintOverzichtPrinter;


PROCEDURE PrintHeadingPrinter;
BEGIN
  WRiteString(OutText,'------------------------------------');
  WRiteString(OutText,'-----------------------------------------');
  WRiteLn(OutText);
  WRiteString(OutText,'|  Time   | Azim | Elev | Doppler | ');
  WRiteString(OutText,'  Dist. |  Height | West | North | /256 |');
  WRiteLn(OutText);
  WRiteString(OutText,'----------+------+------+---------+-');
  WRiteString(OutText,'--------+---------+------+-------+-------')
END PrintHeadingPrinter;



PROCEDURE PrintSlotRegelPrinter;
BEGIN
  WRiteLn(OutText);
  WRiteString(OutText,'------------------------------------');
  WRiteString(OutText,'-----------------------------------------');
  WRiteLn(OutText);
  WRiteString(OutText,'Calculated on an Atari-ST using "ORBIT 1.6" by PA3CVX');
  IF ~LoggingOn THEN
    WRite(OutText,CHR(12))
  END
END PrintSlotRegelPrinter;



PROCEDURE PrintNewDayPrepare(Day:CARDINAL);
VAR DayString: ARRAY[0..9] OF CHAR;
BEGIN
  Assign(OutString,'| ****** ');
  MakeDateString(Day,DayString);
  Concat(OutString,DayString,OutString);
  Concat(OutString,
  '  ******        |         |         |      |       |      |',OutString)
END PrintNewDayPrepare;


PROCEDURE PrintNewOrbitPrepare(Orbit:CARDINAL);
VAR CStr: String;
BEGIN
  Assign(OutString,'| ***** Orbit #');
  ConvertCardinal(Orbit,6,CStr);
  Concat(OutString,CStr,OutString);
  Concat(OutString,
  ' ****        |         |         |      |       |      |',OutString)
END PrintNewOrbitPrepare;


PROCEDURE PrintDataRegelPrepare(Time:REAL);
VAR TString: ARRAY[0..7] OF CHAR;
BEGIN
  MakeTimeString(Time,TString);
  Concat('| ',TString,OutString);
  Concat(OutString,' | ',OutString);
  ConvertInteger(entier(Azimuth+0.5),4,TString);
  Concat(OutString,TString,OutString);
  Concat(OutString,' | ',OutString);
  ConvertInteger(entier(Elevation+0.5),4,TString);
  Concat(OutString,TString,OutString);
  Concat(OutString,' | ',OutString);
  IF DopplerOK THEN
    ConvertReal(DFreq,7,0,TString);
    IF Length(TString)>7 THEN
      Assign(TString,'      0')
    END;
    Concat(OutString,TString,OutString)
  ELSE
    Concat(OutString,'   --  ',OutString)
  END;
  Concat(OutString,' | ',OutString);
  ConvertReal(Distance,7,0,TString);
  IF Length(TString)>7 THEN
    Assign(TString,'     --')
  END;
  Concat(OutString,TString,OutString);
  Concat(OutString,' | ',OutString);
  ConvertReal((HeightC-AardStraal),7,0,TString);
  IF Length(TString)>7 THEN
    Assign(TString,'     --')
  END;
  Concat(OutString,TString,OutString);
  Concat(OutString,' | ',OutString);
  ConvertInteger(entier(WLongitude+0.5),4,TString);
  Concat(OutString,TString,OutString);
  Concat(OutString,' |  ',OutString);
  ConvertInteger(entier(Latitude+0.5),4,TString);
  Concat(OutString,TString,OutString);
  Concat(OutString,' | ',OutString);
  ConvertCardinal(Fase,4,TString);
  Concat(OutString,TString,OutString);
  Concat(OutString,' |',OutString)
END PrintDataRegelPrepare;



PROCEDURE PrintOverzicht(StartTijd,EindTijd:REAL);
BEGIN
  IF ((~OutOnScreen) OR (LoggingOn)) THEN
    PrintOverzichtPrinter(StartTijd,EindTijd)
  END
END PrintOverzicht;



PROCEDURE PrintHeading;
BEGIN
  IF ~OutOnScreen THEN
    PrintHeadingPrinter
  ELSE
    OpenOutWindow;
    WriteOutLine('| Heading');
    IF LoggingOn THEN
      PrintHeadingPrinter
    END
  END
END PrintHeading;


PROCEDURE PrintSlotRegel;
BEGIN
  IF ~OutOnScreen THEN
    PrintSlotRegelPrinter
  ELSE
    WriteOutLine('| Ready');
    IF LoggingOn THEN
      PrintSlotRegelPrinter;
      WRiteLn(OutText);
      WRiteLn(OutText);
      WRiteLn(OutText);
      WRiteLn(OutText);
      WRiteLn(OutText);
      WRiteLn(OutText)
    END
  END
END PrintSlotRegel;


PROCEDURE PrintNewDay(Day:CARDINAL);
BEGIN
  PrintNewDayPrepare(Day);
  IF ~OutOnScreen THEN
    WRiteLn(OutText);
    WRiteString(OutText,OutString)
  ELSE
    WriteOutLine(OutString);
    IF LoggingOn THEN
      WRiteLn(OutText);
      WRiteString(OutText,OutString);
    END
  END
END PrintNewDay;


PROCEDURE PrintNewOrbit(Orbit:CARDINAL);
BEGIN
  PrintNewOrbitPrepare(Orbit);
  IF ~OutOnScreen THEN
    WRiteLn(OutText);
    WRiteString(OutText,OutString)
  ELSE
    WriteOutLine(OutString);
    IF LoggingOn THEN
      WRiteLn(OutText);
      WRiteString(OutText,OutString)
    END
  END
END PrintNewOrbit;


VAR OldElevation,
    OldAzimuth:   REAL;

PROCEDURE PrintDataRegel(Time:REAL);
VAR DoIt: BOOLEAN;
BEGIN
  DoIt:=TRUE;
  IF ~GraphicsOut THEN
    IF ((~ShowLowElevation) & (Elevation<ElevationThres)) THEN
      DoIt:=FALSE
    END;
    IF ((~ShowLittleChanges) &
        (ABS(Elevation-OldElevation)<LittleChangesThres) &
        (ABS(Azimuth-OldAzimuth)<LittleChangesThres)) THEN
      DoIt:=FALSE
    END
  END;
  IF DoIt THEN
    OldElevation:=Elevation;
    OldAzimuth:=Azimuth;
    PrintDataRegelPrepare(Time);
    IF ~OutOnScreen THEN
      WRiteLn(OutText);
      WRiteString(OutText,OutString)
    ELSE
      WriteOutLine(OutString);
      IF LoggingOn THEN
        WRiteLn(OutText);
        WRiteString(OutText,OutString)
      END
    END
  END
END PrintDataRegel;


END OOutput.
