IMPLEMENTATION MODULE OOutWindow;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


FROM SYSTEM             IMPORT ADR;
FROM Strings            IMPORT String,
                               Assign,
                               Copy,
                               Concat,
                               Length;
FROM InOut              IMPORT OpenInputFile,
                               CloseInput,
                               Done;
FROM GEMDOS             IMPORT Open,
                               Read,
                               Close;
FROM M2Conversions      IMPORT ConvertToInteger;

(********************** IMPORT from ORBIT specific libraries *)
FROM OGEMGlobal         IMPORT MenuTreeAddr,
                               WorkstationHandle,
                               CharWidth,
                               CharHeight,
                               Message,
                               GraphicsOut,
                               Touched,
                               PRINTER,
                               SCREEN,
                               GRAPHICS;
FROM OXBIOS             IMPORT ScreenAddress;
FROM OOutSave           IMPORT SaveOutput,
                               UpdateOutput;
FROM OFiler             IMPORT SatRec;

(*********************) IMPORT GEMAESbase;
FROM AESWindows         IMPORT WindowGet,
                               WindowCalc,
                               WindowCreate,
                               WindowClose,
                               WindowDelete,
                               WindowSet,
                               WindowOpen;
FROM AESGraphics        IMPORT GrafMouseKeyboardState,
                               GrafShrinkBox,
                               GrafGrowBox;
FROM AESForms           IMPORT FormAlert;
FROM AESMenus           IMPORT MenuItemEnable;

(*********************) IMPORT GEMVDIbase;
FROM VDIControls        IMPORT SetClipping;
FROM VDIOutputs         IMPORT FillRectangle,
                               GraphicText;
FROM VDIEscapes         IMPORT RemoveCursor,
                               DisplayCursor;
FROM VDIRasters         IMPORT CopyRasterOpaque;




CONST BufSize= 100;

VAR XTotal,YTotal,WTotal,HTotal,
    XWork,YWork,WWork,HWork,
    XOld,YOld,WOld,HOld,
    HomeX,HomeY,
    MouseX,MouseY,MouseButton,
    MouseKeyboardState,
    TextX,TextY,
    CrossX,CrossY,
    VSlidPos,
    VSlidRelSize,
    InfoLines,
    VisibleLines,
    StartLine,
    BufPnt,
    Index,
    WindowComponents,
    FileHandle:                         INTEGER;
    OrbitString,
    DayString,
    WindowName:                         String;
    PxyArray:                           GEMVDIbase.PxyArrayType;
    NrOfBytes:                          LONGCARD;
    MFDBcross,
    MFDBsave,
    MFDBmem,
    MFDBscreen:                         ARRAY[1..10] OF INTEGER;
    BigPxy:                             GEMVDIbase.BigPxyArrayType;
    CrossOnScreen,
    CloseOk,
    WindowFulled:                       BOOLEAN;
    OrbitBuf,
    DayBuf,
    Buffer:                             ARRAY[0..BufSize-1] OF String;
    Cross,CrossSave:                    ARRAY[0..10] OF INTEGER;




PROCEDURE ComputeVSlidPos;
BEGIN
  IF VisibleLines<InfoLines THEN
    VSlidPos:=((100*StartLine) DIV (InfoLines-VisibleLines))*10
  ELSE
    VSlidPos:=1
  END
END ComputeVSlidPos;



PROCEDURE InitCross;
BEGIN
  Cross[0]:=1010H;
  Cross[1]:=3838H;
  Cross[2]:=1C70H;
  Cross[3]:=0FE0H;
  Cross[4]:=07C0H;
  Cross[5]:=07C0H;
  Cross[6]:=07C0H;
  Cross[7]:=0FE0H;
  Cross[8]:=1C70H;
  Cross[9]:=3838H;
  Cross[10]:=1010H;
  MFDBcross[1]:=INTEGER(LONGCARD(ADR(Cross[0])) DIV 10000H);
  MFDBcross[2]:=INTEGER(LONGCARD(ADR(Cross[0])) MOD 10000H);
  MFDBcross[3]:=16;
  MFDBcross[4]:=11;
  MFDBcross[5]:=1;
  MFDBcross[6]:=0;
  MFDBcross[7]:=1;
  MFDBsave[1]:=INTEGER(LONGCARD(ADR(CrossSave[0])) DIV 10000H);
  MFDBsave[2]:=INTEGER(LONGCARD(ADR(CrossSave[0])) MOD 10000H);
  MFDBsave[3]:=16;
  MFDBsave[4]:=11;
  MFDBsave[5]:=1;
  MFDBsave[6]:=0;
  MFDBsave[7]:=1
END InitCross;



PROCEDURE PlaceCross(Index:INTEGER);
(* West and north from Buffer[Index] *)
VAR West,North: INTEGER;
    S: String;
    Ok: BOOLEAN;
BEGIN
  Copy(Buffer[Index],57,3,S);
  ConvertToInteger(S,Ok,West);
  IF ((~Ok) OR (West<0) OR (West>360)) THEN
    Touched:=FormAlert(1,"[1][ | 'West' has wrong format!][Continue]");
    West:=0
  END;
  IF West>180 THEN
    West:=West-360
  END;
  CrossX:=XWork+301-((West*3+1) DIV 2);
  Copy(Buffer[Index],65,3,S);
  ConvertToInteger(S,Ok,North);
  IF ((~Ok) OR (North<-90) OR (North>90)) THEN
    Touched:=FormAlert(1,"[1][ | 'North' has wrong format!][Continue]");
    North:=0
  END;
  CrossY:=YWork+160-((North*144+72) DIV 90);
  BigPxy[0]:=CrossX;
  BigPxy[1]:=CrossY;
  BigPxy[2]:=CrossX+15;
  BigPxy[3]:=CrossY+10;
  BigPxy[4]:=0;
  BigPxy[5]:=0;
  BigPxy[6]:=15;
  BigPxy[7]:=10;
  CopyRasterOpaque(WorkstationHandle,3,BigPxy,
                        ADR(MFDBscreen),ADR(MFDBsave));
  BigPxy[0]:=0;
  BigPxy[1]:=0;
  BigPxy[2]:=15;
  BigPxy[3]:=10;
  BigPxy[4]:=CrossX;
  BigPxy[5]:=CrossY;
  BigPxy[6]:=CrossX+14;
  BigPxy[7]:=CrossY+10;
  CopyRasterOpaque(WorkstationHandle,7,BigPxy,
                        ADR(MFDBcross),ADR(MFDBscreen));
  CrossOnScreen:=TRUE
END PlaceCross;



PROCEDURE RemoveCross;
BEGIN
  BigPxy[0]:=0;
  BigPxy[1]:=0;
  BigPxy[2]:=15;
  BigPxy[3]:=10;
  BigPxy[4]:=CrossX;
  BigPxy[5]:=CrossY;
  BigPxy[6]:=CrossX+15;
  BigPxy[7]:=CrossY+10;
  CopyRasterOpaque(WorkstationHandle,1,BigPxy,
                        ADR(MFDBsave),ADR(MFDBscreen));
  CrossOnScreen:=FALSE
END RemoveCross;



PROCEDURE WriteTextHeading;
VAR Stri: String;
BEGIN
  TextX:=XWork;
  TextY:=YWork+11;
  Assign(Stri,
'----------+------+------+---------+---------+---------+------+-------+------- ');
  GraphicText(WorkstationHandle,TextX,TextY,Stri);
  Assign(Stri,
'|  Time   | Azim | Elev | Doppler |  Dist.  |  Height | West | North | /256 | ');
  TextY:=TextY+CharHeight-2;
  GraphicText(WorkstationHandle,TextX,TextY,Stri);
  TextY:=TextY+CharHeight;
  Assign(Stri,
'----------+------+------+---------+---------+---------+------+-------+------- ');
  GraphicText(WorkstationHandle,TextX,TextY,Stri);
  TextY:=TextY+CharHeight-3
END WriteTextHeading;



PROCEDURE WriteTextEmptyLines(NrOfLines:CARDINAL);
VAR X: CARDINAL;
    Stri: String;
BEGIN
  Assign(Stri,
'|         |      |      |         |         |         |      |       |      | ');
  FOR X:=1 TO NrOfLines DO
    GraphicText(WorkstationHandle,TextX,TextY,Stri);
    TextY:=TextY+CharHeight
  END
END WriteTextEmptyLines;



PROCEDURE WriteGrafPicture;
BEGIN
  BigPxy[0]:=0;
  BigPxy[1]:=0;
  BigPxy[2]:=WWork-1;
  BigPxy[3]:=360;
  BigPxy[4]:=XWork;
  BigPxy[5]:=YWork;
  BigPxy[6]:=XWork+WWork-1;
  BigPxy[7]:=YWork+HWork-1;
  CopyRasterOpaque(WorkstationHandle,3,BigPxy,
                      ADR(MFDBmem),ADR(MFDBscreen))
END WriteGrafPicture;



PROCEDURE PrepareForNewInfo;
BEGIN
  GrafMouseKeyboardState(MouseX,MouseY,MouseButton,MouseKeyboardState);
  RemoveCursor(WorkstationHandle);
  BufPnt:=0;
  InfoLines:=0;
  PxyArray[0]:=XWork;
  PxyArray[1]:=YWork;
  PxyArray[2]:=XWork+WWork-1;
  PxyArray[3]:=YWork+HWork-1;
  SetClipping(WorkstationHandle,1,PxyArray);
  IF ~GraphicsOut THEN
    WriteTextHeading;
    WriteTextEmptyLines(20);
  ELSE
    WriteGrafPicture;
    CrossOnScreen:=FALSE
  END
END PrepareForNewInfo;



PROCEDURE ScanReady;
VAR Ch: CHAR;
BEGIN
  DisplayCursor(WorkstationHandle,MouseX,MouseY);
  IF GraphicsOut THEN
    VisibleLines:=1;
    StartLine:=InfoLines-1
  ELSE
    IF InfoLines>20 THEN
      VisibleLines:=20;
      StartLine:=InfoLines-20
    ELSE
      VisibleLines:=InfoLines;
      StartLine:=0
    END
  END;
  ComputeVSlidPos;
  WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
  VSlidRelSize:=((VisibleLines*100) DIV InfoLines)*10;
  WindowSet(OutWindowHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
  SaveOutput
END ScanReady;



PROCEDURE WriteGrafOrbit(S:String);
  PROCEDURE DoOut;
  BEGIN
    GraphicText(WorkstationHandle,XWork+84+3*CharWidth,YWork+211,S);
  END DoOut;
BEGIN
  IF CrossOnScreen THEN
    RemoveCross;
    DoOut;
    PlaceCross(BufPnt-1)
  ELSE
    DoOut
  END
END WriteGrafOrbit;



PROCEDURE NewOrbit(Str: String);
VAR HStr: String;
BEGIN
  IF ~GraphicsOut THEN
    Assign(Buffer[BufPnt],Str);
    BufPnt:=(BufPnt+1) MOD BufSize;
    InfoLines:=InfoLines+1;
    IF InfoLines>BufSize THEN
      InfoLines:=BufSize
    END;
    TextX:=XWork;
    TextY:=YWork+HWork-3;
    ShiftText;
    GraphicText(WorkstationHandle,TextX,TextY,Str)
  ELSE
    Assign(OrbitString,Str);
    Copy(Str,15,6,HStr);
    WriteGrafOrbit(HStr)
  END
END NewOrbit;



PROCEDURE WriteGrafDay(S:String);
  PROCEDURE DoOut;
  BEGIN
    GraphicText(WorkstationHandle,XWork+84,YWork+195,S);
    GraphicText(WorkstationHandle,XWork+44,YWork+243,SatRec.Naam)
  END DoOut;
BEGIN
  IF CrossOnScreen THEN
    RemoveCross;
    DoOut;
    PlaceCross(BufPnt-1)
  ELSE
    DoOut
  END
END WriteGrafDay;



PROCEDURE NewDay(Str: String);
VAR HStr: String;
BEGIN
  IF ~GraphicsOut THEN
    Assign(Buffer[BufPnt],Str);
    BufPnt:=(BufPnt+1) MOD BufSize;
    InfoLines:=InfoLines+1;
    IF InfoLines>BufSize THEN
      InfoLines:=BufSize
    END;
    TextX:=XWork;
    TextY:=YWork+HWork-3;
    ShiftText;
    GraphicText(WorkstationHandle,TextX,TextY,Str)
  ELSE
    Assign(DayString,Str);
    Copy(Str,9,9,HStr);
    WriteGrafDay(HStr)
  END
END NewDay;


PROCEDURE ShiftText;
BEGIN
  BigPxy[0]:=XWork;
  BigPxy[1]:=YWork+HWork-1-19*CharHeight;
  BigPxy[2]:=XWork+WWork-1;
  BigPxy[3]:=YWork+HWork-1;
  BigPxy[4]:=XWork;
  BigPxy[5]:=YWork+HWork-1-20*CharHeight;
  BigPxy[6]:=XWork+WWork-1;
  BigPxy[7]:=YWork+HWork-1-CharHeight;
  CopyRasterOpaque(WorkstationHandle,3,BigPxy,
                             ADR(MFDBscreen),ADR(MFDBscreen))
END ShiftText;


PROCEDURE WriteOutLine(Str: String);
BEGIN
  CASE Str[2] OF
    'H': PrepareForNewInfo      |
    'R': ScanReady              |
    '*': IF Str[8]='O' THEN
           NewOrbit(Str)
         ELSE
           NewDay(Str)
         END                    |
  ELSE
    Assign(Buffer[BufPnt],Str);
    TextX:=XWork;
    TextY:=YWork+HWork-3;
    IF ~GraphicsOut THEN
      ShiftText
    ELSE
      Assign(OrbitBuf[BufPnt],OrbitString);
      Assign(DayBuf[BufPnt],DayString);
      IF CrossOnScreen THEN
        RemoveCross
      END;
      PlaceCross(BufPnt)
    END;
    BufPnt:=(BufPnt+1) MOD BufSize;
    InfoLines:=InfoLines+1;
    IF InfoLines>BufSize THEN
      InfoLines:=BufSize
    END;
    GraphicText(WorkstationHandle,TextX,TextY,Str)
  END
END WriteOutLine;



PROCEDURE UpdateGrafDay(Index:INTEGER);
VAR HStr: String;
BEGIN
  Copy(DayBuf[Index],9,9,HStr);
  GraphicText(WorkstationHandle,XWork+84,YWork+195,HStr);
  GraphicText(WorkstationHandle,XWork+44,YWork+243,SatRec.Naam)
END UpdateGrafDay;



PROCEDURE UpdateGrafOrbit(Index:INTEGER);
VAR HStr: String;
BEGIN
  Copy(OrbitBuf[Index],15,6,HStr);
  GraphicText(WorkstationHandle,XWork+84+3*CharWidth,YWork+211,HStr)
END UpdateGrafOrbit;



PROCEDURE UpdateOutWindowInfo;
BEGIN
  PxyArray[0]:=XWork;
  PxyArray[1]:=YWork;
  PxyArray[2]:=XWork+WWork-1;
  PxyArray[3]:=YWork+HWork-1;
  SetClipping(WorkstationHandle,1,PxyArray);
  IF InfoLines=BufSize THEN
    Index:=(BufPnt+StartLine) MOD BufSize
  ELSE
    Index:=StartLine
  END;
  TextX:=XWork;
  IF ~GraphicsOut THEN
    IF InfoLines>20 THEN
      TextY:=YWork+3*CharHeight+6;
      WHILE TextY<(YWork+HWork) DO
        GraphicText(WorkstationHandle,TextX,TextY,Buffer[Index]);
        Index:=(Index+1) MOD BufSize;
        TextY:=TextY+CharHeight
      END
    END
  ELSE
    TextY:=YWork+HWork-3;
    GraphicText(WorkstationHandle,TextX,TextY,Buffer[Index]);
    RemoveCross;
    UpdateGrafDay(Index);
    UpdateGrafOrbit(Index);
    PlaceCross(Index)
  END;
  SaveOutput
END UpdateOutWindowInfo;



PROCEDURE OpenOutWindow;
VAR Dum: INTEGER;
BEGIN
  IF ~OutWindowIsOpen THEN
    MenuItemEnable(MenuTreeAddr,PRINTER,0);
    MenuItemEnable(MenuTreeAddr,SCREEN,0);
    MenuItemEnable(MenuTreeAddr,GRAPHICS,0);
    WindowGet(0,GEMAESbase.WorkXYWH,XTotal,YTotal,WTotal,HTotal);
    WindowComponents:=GEMAESbase.Name+
                      GEMAESbase.Closer+
                      GEMAESbase.Mover+
                      GEMAESbase.UpArrow+
                      GEMAESbase.DownArrow+
                      GEMAESbase.VertSlide+
                      GEMAESbase.Fuller;
    OutWindowHandle:=WindowCreate(WindowComponents,
                                  XTotal,YTotal,WTotal,HTotal);
    WindowName:=" Output ";
    WindowSet(OutWindowHandle,GEMAESbase.WindowName,
              INTEGER(LONGCARD(ADR(WindowName)) DIV 10000H),
              INTEGER(LONGCARD(ADR(WindowName)) MOD 10000H),0,0);
    VSlidRelSize:=1000;
    WindowSet(OutWindowHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
    VSlidPos:=1;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
    WindowCalc(1,WindowComponents,
               XTotal,YTotal,WTotal,HTotal,
               XWork,YWork,WWork,HWork);
    HomeX:=XWork;
    HomeY:=YWork;
    XOld:=XTotal;
    YOld:=YTotal;
    WOld:=WTotal;
    HOld:=HTotal;
    GrafGrowBox(XTotal+(WTotal DIV 2),YTotal+(HTotal DIV 2),1,1,
              XTotal,YTotal,WTotal,HTotal);
    WindowOpen(OutWindowHandle,XTotal,YTotal,WTotal,HTotal);
    WindowFulled:=TRUE;
    OutWindowIsOpen:=TRUE
  ELSE
    WindowSet(OutWindowHandle,GEMAESbase.Top,OutWindowHandle,Dum,Dum,Dum);
    IF ~WindowFulled THEN
      OutWindowFulled
    END
  END
END OpenOutWindow;



PROCEDURE OutWindowClosed;
BEGIN
  WindowClose(OutWindowHandle);
  GrafShrinkBox(XWork+(WWork DIV 2),YWork+(HWork DIV 2),1,1,
              XWork,YWork,WWork,HWork);
  WindowDelete(OutWindowHandle);
  OutWindowIsOpen:=FALSE;
  MenuItemEnable(MenuTreeAddr,PRINTER,1);
  MenuItemEnable(MenuTreeAddr,SCREEN,1);
  MenuItemEnable(MenuTreeAddr,GRAPHICS,1)
END OutWindowClosed;



PROCEDURE NewWindow(X,Y,W,H: INTEGER);
BEGIN
  WindowSet(OutWindowHandle,GEMAESbase.CurrXYWH,X,Y,W,H);
  IF ((X#XTotal) OR (Y#YTotal)) THEN
    XOld:=X;
    YOld:=Y;
    WOld:=W;
    HOld:=H;
    WindowFulled:=FALSE
  ELSE
    WindowFulled:=TRUE
  END;
  WindowGet(OutWindowHandle,GEMAESbase.WorkXYWH,XWork,YWork,WWork,HWork);
  UpdateOutput
END NewWindow;



PROCEDURE AlertNotFulled;
VAR Dum:INTEGER;
BEGIN
  Dum:=FormAlert(1,'[1][Output window must be totally|visible to perform this|function! | |(Use "Window-Fuller")][Ok]')
END AlertNotFulled;



PROCEDURE OutWindowVertSlided;
BEGIN
  IF ((HomeX=XWork) & (HomeY=YWork)) THEN
    VSlidPos:=Message[4];
    StartLine:=((InfoLines-VisibleLines)*(VSlidPos DIV 10)) DIV 100;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
    UpdateOutWindowInfo
  ELSE
    AlertNotFulled
  END
END OutWindowVertSlided;



PROCEDURE OutWindowArrowed;

  PROCEDURE LineUp;
  BEGIN
    StartLine:=StartLine-1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineUp;

  PROCEDURE LineDown;
  BEGIN
    StartLine:=StartLine+1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineDown;

  PROCEDURE PageUp;
  BEGIN
    StartLine:=StartLine-VisibleLines+1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageUp;

  PROCEDURE PageDown;
  BEGIN
    StartLine:=StartLine+VisibleLines-1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(OutWindowHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageDown;

BEGIN
  IF ((HomeX=XWork) & (HomeY=YWork)) THEN
    CASE Message[4] OF
      0: PageUp           |
      1: PageDown         |
      2: LineUp           |
      3: LineDown         |
    ELSE
      Touched:=FormAlert(1,
                    '[1][Unknown arrow function! |(Output window)][Ok]')
    END;
    UpdateOutWindowInfo
  ELSE
    AlertNotFulled
  END
END OutWindowArrowed;



PROCEDURE OutWindowFulled;
BEGIN
  IF WindowFulled THEN
    GrafShrinkBox(XOld,YOld,WOld,HOld,XTotal,YTotal,WTotal,HTotal);
    NewWindow(XOld,YOld,WOld,HOld)
  ELSE
    GrafGrowBox(XOld,YOld,WOld,HOld,XTotal,YTotal,WTotal,HTotal);
    NewWindow(XTotal,YTotal,WTotal,HTotal)
  END
END OutWindowFulled;



PROCEDURE MessageOutWindow;
BEGIN
  CASE Message[0] OF
    GEMAESbase.WindowRedraw:      UpdateOutput                            |
    GEMAESbase.WindowTopped:      WindowSet(OutWindowHandle,GEMAESbase.Top,
                                            OutWindowHandle,0,0,0)        |
    GEMAESbase.WindowClosed:      OutWindowClosed                         |
    GEMAESbase.WindowArrowed:     OutWindowArrowed                        |
    GEMAESbase.WindowVertSlided:  OutWindowVertSlided                     |
    GEMAESbase.WindowMoved:       NewWindow(Message[4],Message[5],
                                            Message[6],Message[7])        |
    GEMAESbase.WindowFulled:      OutWindowFulled                         |
  ELSE
    (* Do Nothing *)
  END
END MessageOutWindow;


PROCEDURE InitOutWindow;
TYPE Picture= ARRAY[0..32000] OF CHAR;
VAR Pict: Picture;
    Pic: POINTER TO Picture;
BEGIN
  Pic:=ADR(Pict);
  OutWindowIsOpen:=FALSE;
  MFDBmem[1]:=INTEGER((LONGCARD(Pic)+34) DIV 10000H);
  MFDBmem[2]:=INTEGER((LONGCARD(Pic)+34) MOD 10000H);
  MFDBmem[3]:=640;
  MFDBmem[4]:=365;
  MFDBmem[5]:=40;
  MFDBmem[6]:=0;
  MFDBmem[7]:=1;
  MFDBscreen[1]:=INTEGER(LONGCARD(ScreenAddress()) DIV 10000H);
  MFDBscreen[2]:=INTEGER(LONGCARD(ScreenAddress()) MOD 10000H);
  MFDBscreen[3]:=640;
  MFDBscreen[4]:=400;
  MFDBscreen[5]:=40;
  MFDBscreen[6]:=0;
  MFDBscreen[7]:=1;
  OpenInputFile('ORBIT.DAT\ORBIT.PIC');
  IF ~Done THEN
    Touched:=FormAlert(1,'[3][Picture file not found!| |Program ORBIT.PRG needs the|picture file ORBIT.PIC!][Abort]');
    HALT
  END;
  CloseInput;
  Open('ORBIT.DAT\ORBIT.PIC',0,FileHandle);
  NrOfBytes:=31000;
  Read(FileHandle,NrOfBytes,Pic);
  CloseOk:=Close(FileHandle)
END InitOutWindow;


BEGIN (* Initialisation of OOutWindow *)
  InitOutWindow;
  InitCross
END OOutWindow.
