IMPLEMENTATION MODULE OParamIn;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


(********************** IMPORT from non GEM libraries *)
FROM SYSTEM             IMPORT ADDRESS;
FROM Strings            IMPORT String,
                               Concat,
                               Copy,
                               Assign,
                               Length;
FROM M2Conversions      IMPORT ConvertToCardinal,
                               ConvertToReal,
                               ConvertCardinal,
                               ConvertReal;

(********************** IMPORT from specific ORBIT libraries *)
FROM OGEMGlobal         IMPORT SCANPARA,
                               START,
                               ERRFIELD,
                               SETPARA,
                               YEARSET,
                               MONTHSET,
                               DAYSET,
                               HOURSSET,
                               MINUTSET,
                               DURHRSET,
                               DURMISET,
                               STEPSET,
                               SETOK,
                               SETCANCE,
                               Touched,
                               MenuTreeAddr,
                               InputErrTreeAddr,
                               SetParaTreeAddr,
                               TedInfoAddr,
                               TextAddr,
                               SatelliteActive,
                               ScanParamsActive,
                               HandleForm;
FROM OrbGlobal          IMPORT M1e,
                               StartYear,
                               StartMonth,
                               StartDay,
                               StartHours,
                               StartMinutes,
                               DurationHours,
                               DurationMinutes,
                               TimeStep;
FROM OClock             IMPORT TimeRecord,
                               ReadTime;
FROM OScanInfo          IMPORT UpdateScanInfo,
                               ScanInfoIsOpen;

(*********************) IMPORT GEMAESbase;
FROM AESMenus           IMPORT MenuItemEnable;



VAR TempStartYear,
    TempStartMonth,
    TempStartDay,
    TempStartHours,
    TempStartMinutes,
    TempDurationHours,
    TempDurationMinutes: CARDINAL;
    TempTimeStep:        REAL;
    TimeRec:             TimeRecord;



PROCEDURE DisplayErrorField(VAR Str: ARRAY OF CHAR);
BEGIN
  TedInfoAddr:=ADDRESS(InputErrTreeAddr^[ERRFIELD].ObSpec);
  TextAddr:=TedInfoAddr^.TePText;
  Assign(TextAddr^,Str);
  HandleForm(InputErrTreeAddr,0)
END DisplayErrorField;



PROCEDURE ConvToStrings;
VAR Str: String;

  PROCEDURE CardToString(Item:INTEGER; Value:CARDINAL);
  VAR Str1: String;
  BEGIN
    TedInfoAddr:=ADDRESS(SetParaTreeAddr^[Item].ObSpec);
    TextAddr:=TedInfoAddr^.TePText;
    ConvertCardinal(Value,1,Str1);
    Concat('  ',Str1,Str);
    Copy(Str,Length(Str)-2,2,TextAddr^)
  END CardToString;

BEGIN (* ConvToString *)
  CardToString(YEARSET,StartYear);
  CardToString(MONTHSET,StartMonth);
  CardToString(DAYSET,StartDay);
  CardToString(HOURSSET,StartHours);
  CardToString(MINUTSET,StartMinutes);
  CardToString(DURHRSET,DurationHours);
  CardToString(DURMISET,DurationMinutes);
  TedInfoAddr:=ADDRESS(SetParaTreeAddr^[STEPSET].ObSpec);
  TextAddr:=TedInfoAddr^.TePText;
  ConvertReal(TimeStep,15,-10,Str);
  Assign(TextAddr^,Str)
END ConvToStrings;



PROCEDURE ConvToValues(VAR ErrField:INTEGER);
VAR DoneOk: BOOLEAN;

  PROCEDURE CardConvert(Item:INTEGER; VAR Dest:CARDINAL);
  BEGIN
    TedInfoAddr:=ADDRESS(SetParaTreeAddr^[Item].ObSpec);
    TextAddr:=TedInfoAddr^.TePText;
    IF Length(TextAddr^)=0 THEN
      Dest:=0;
      DoneOk:=TRUE
    ELSE
      ConvertToCardinal(TextAddr^,DoneOk,Dest)
    END
  END CardConvert;

  PROCEDURE RealConvert(Item:INTEGER; VAR Dest:REAL);
  BEGIN
    TedInfoAddr:=ADDRESS(SetParaTreeAddr^[Item].ObSpec);
    TextAddr:=TedInfoAddr^.TePText;
    IF Length(TextAddr^)=0 THEN
      Dest:=0.0;
      DoneOk:=TRUE
    ELSE
      ConvertToReal(TextAddr^,DoneOk,Dest)
    END
  END RealConvert;

BEGIN (* ConvToValues *)
  CardConvert(YEARSET,TempStartYear);
  IF (~DoneOk OR (TempStartYear<80)) THEN
    DisplayErrorField('Start Year.   (Must be >=1980)');
    ErrField:=YEARSET
  ELSE
    CardConvert(MONTHSET,TempStartMonth);
    IF (~DoneOk OR (TempStartMonth=0) OR (TempStartMonth>12)) THEN
      DisplayErrorField('Start Month.   (Must be 1..12)');
      ErrField:=MONTHSET
    ELSE
      CardConvert(DAYSET,TempStartDay);
      IF (~DoneOk OR
          (TempStartDay=0) OR
          ( ( (M1e[TempStartMonth]+TempStartDay)>M1e[TempStartMonth+1]) &
           ~((TempStartMonth=2) & (TempStartDay=29)) ) ) THEN
        DisplayErrorField('Start Day.');
        ErrField:=DAYSET
      ELSE
        CardConvert(HOURSSET,TempStartHours);
        IF (~DoneOk OR (TempStartHours>23)) THEN
          DisplayErrorField('Start Hours.   (Must be 0..23)');
          ErrField:=HOURSSET
        ELSE
          CardConvert(MINUTSET,TempStartMinutes);
          IF (~DoneOk OR (TempStartMinutes>59)) THEN
            DisplayErrorField('Start Minutes.  (Must be 0..59)');
            ErrField:=MINUTSET
          ELSE
            CardConvert(DURHRSET,TempDurationHours);
            IF ~DoneOk THEN
              DisplayErrorField('Duration Hours.');
              ErrField:=DURHRSET
            ELSE
              CardConvert(DURMISET,TempDurationMinutes);
              IF (~DoneOk OR (TempDurationMinutes>59)) THEN
                DisplayErrorField('Duration Minutes.  (Must be 0..59)');
                ErrField:=DURMISET
              ELSE
                RealConvert(STEPSET,TempTimeStep);
                IF ~DoneOk THEN
                  DisplayErrorField('Time Step.');
                  ErrField:=STEPSET
                ELSE
                  ErrField:=0
                END
              END
            END
          END
        END
      END
    END
  END
END ConvToValues;



PROCEDURE GetScanParams;
VAR First,ErrFieldNr: INTEGER;
    Ready: BOOLEAN;
BEGIN
  ConvToStrings;
  First:=YEARSET;
  Ready:=FALSE;
  REPEAT
    HandleForm(SetParaTreeAddr,First);
    IF Touched=SETCANCE THEN
      Ready:=TRUE
    ELSE
      ConvToValues(ErrFieldNr);
      IF ErrFieldNr=0 THEN
        Ready:=TRUE
      ELSE
        First:=ErrFieldNr
      END
    END
  UNTIL Ready;
  IF Touched=SETOK THEN
    StartYear:=TempStartYear;
    StartMonth:=TempStartMonth;
    StartDay:=TempStartDay;
    StartHours:=TempStartHours;
    StartMinutes:=TempStartMinutes;
    DurationHours:=TempDurationHours;
    DurationMinutes:=TempDurationMinutes;
    TimeStep:=TempTimeStep;
    IF ~ScanInfoIsOpen THEN
      MenuItemEnable(MenuTreeAddr,SCANPARA,1)
    END;
    IF SatelliteActive THEN
      MenuItemEnable(MenuTreeAddr,START,1)
    END;
    ScanParamsActive:=TRUE;
    IF ScanInfoIsOpen THEN
      UpdateScanInfo
    END
  END
END GetScanParams;


BEGIN (* OParamIn *)
  ReadTime(TimeRec);
  StartYear:=TimeRec.Year;
  StartMonth:=TimeRec.Month;
  StartDay:=TimeRec.Day;
  StartHours:=TimeRec.Hour;
  StartMinutes:=TimeRec.Min;
  DurationHours:=1;
  DurationMinutes:=0;
  TimeStep:=1.0
END OParamIn.
