MODULE Orbit;

(* GEM version of ORBIT        Version 1.6 *)
(* By: Harrie F.A. de Leeuw, November 1990 *)


(* =================== *)
(* Main program module *)
(* =================== *)


(*********************  IMPORT from not-GEM system libraries *)
FROM SYSTEM             IMPORT ADR,
                               ADDRESS;
FROM InOut              IMPORT OpenInputFile,
                               CloseInput,
                               Done;

(*********************  IMPORT from specific ORBIT libraries *)
FROM OGEMGlobal         IMPORT MENU,
                               DESK,
                               FILE,
                               COMPUTE,
                               OUTPUT,
                               INFO,
                               TIMEDATE,
                               ABOUT,
                               LOADSA,
                               SAVESA,
                               NEWSA,
                               CHANGESA,
                               DELETESA,
                               CHANGEST,
                               QUIT,
                               ORBINFO,
                               SATELIN,
                               STATIN,
                               INPUTERR,
                               SETPARA,
                               SYSTIME,
                               TRACK,
                               START,
                               SETSCAN,
                               NOTSAVED,
                               INFOOK,
                               SCANPARA,
                               SALOADED,
                               STLOADED,
                               SCREEN,
                               GRAPHICS,
                               PRINTER,
                               SETTIME,
                               SETELEVT,
                               SETLITTT,
                               SHOWELEV,
                               SETELEVA,
                               SHOWLITT,
                               SETLITTL,
                               HALTEDT,
                               TLOG,
                               ELOG,
                               LoggingOn,
                               OutOnScreen,
                               GraphicsOut,
                               Message,
                               SatelliteNotSaved,
                               MenuTreeAddr,
                               SatelInTreeAddr,
                               StatInTreeAddr,
                               SetParaTreeAddr,
                               SetElevThresTree,
                               SetLittleThresTree,
                               HaltedTree,
                               InputErrTreeAddr,
                               NotSavedTreeAddr,
                               OrbInfoTreeAddr,
                               TimeDateTreeAddr,
                               LogTreeAddr,
                               CharWidth,
                               CharHeight,
                               ApId,
                               WorkstationHandle,
                               HandleForm;
FROM OrbGlobal          IMPORT ShowLowElevation,
                               ShowLittleChanges;
FROM OScanInfo          IMPORT ScanInfoHandle,
                               OpenScanInfo,
                               UpdateScanInfo,
                               MessageScanInfo,
                               ScanInfoIsOpen;
FROM OSatelInfo         IMPORT SatelInfoHandle,
                               OpenSatelInfo,
                               UpdateSatelInfo,
                               MessageSatelInfo,
                               SatelInfoIsOpen;
FROM OStatInfo          IMPORT StatInfoHandle,
                               OpenStatInfo,
                               UpdateStatInfo,
                               MessageStatInfo,
                               StatInfoIsOpen;
FROM OFiler             IMPORT LoadSatellite,
                               SaveSatellite,
                               NewSatellite,
                               ChangeSatellite,
                               DeleteSatellite,
                               ChangeStation,
                               LoadStation,
                               SatNotSavedDialogue;
FROM OOutWindow         IMPORT OutWindowHandle,
                               MessageOutWindow,
                               OutWindowIsOpen;
FROM OOutput            IMPORT OutToPRN,
                               CloseOutToPRN,
                               OutToFile,
                               CloseOutToFile;
FROM OParamIn           IMPORT GetScanParams;
FROM OCompute           IMPORT Track,
                               Scan;
FROM OTimeDate          IMPORT SetSystemTimeAndDate;
FROM OXBIOS             IMPORT ScreenResolution;
FROM OThresIn           IMPORT GetElevationThres,
                               GetLittleChangesThres;

(*********************) IMPORT GEMAESbase;
FROM AESGraphics        IMPORT GrafHandle,
                               GrafMouse;
FROM AESApplications    IMPORT ApplInitialise,
                               ApplExit;
FROM AESResources       IMPORT ResourceLoad,
                               ResourceGetAddr,
                               ResourceFree;
FROM AESMenus           IMPORT MenuBar,
                               MenuTitleNormal,
                               MenuItemCheck,
                               MenuItemEnable;
FROM AESEvents          IMPORT EventMessage;
FROM AESForms           IMPORT FormAlert;

(*********************) IMPORT GEMVDIbase;
FROM VDIControls        IMPORT OpenVirtualWorkstation,
                               CloseVirtualWorkstation;




VAR PhysHandle,
    CharBoxWidth, CharBoxHeight,
    Dummy:                          INTEGER;
    WorkIn:                         GEMVDIbase.VDIWorkInType;
    WorkOut:                        GEMVDIbase.VDIWorkOutType;
    StopProgram:                    BOOLEAN;




PROCEDURE Initialise():BOOLEAN;
VAR Count: CARDINAL;
    InitBoolean: BOOLEAN;
    DummyAddress: ADDRESS;
BEGIN
  ApId:=ApplInitialise();
  PhysHandle:=GrafHandle(CharWidth,CharHeight,CharBoxWidth,CharBoxHeight);
  FOR Count:=1 TO 8 DO
    WorkIn[Count]:=1
  END;
  WorkIn[9]:=0;  (* fill color white *)
  WorkIn[10]:=2;
  WorkstationHandle:=PhysHandle;
  OpenVirtualWorkstation(WorkIn,WorkstationHandle,WorkOut);
  IF ScreenResolution()<>2 THEN
    GrafMouse(GEMAESbase.Arrow,DummyAddress);
    Dummy:=FormAlert(1,'[3][Sorry!  High resolution only!][Abort]');
    InitBoolean:=FALSE
  ELSE
    OpenInputFile('ORBIT.DAT\ORBIT.RSC');(* Test existence  resource file *)
    IF NOT(Done) THEN
      GrafMouse(GEMAESbase.Arrow,DummyAddress);
      Dummy:=FormAlert(1,'[3][Resource file not found!| |Program ORBIT.PRG needs the|resource file ORBIT.RSC!][Abort]');
      InitBoolean:=FALSE
    ELSE
      CloseInput;
      ResourceLoad('ORBIT.DAT\ORBIT.RSC');
      ResourceGetAddr(GEMAESbase.RTree,MENU,MenuTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,SATELIN,SatelInTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,STATIN,StatInTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,SETPARA,SetParaTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,SETELEVT,SetElevThresTree);
      ResourceGetAddr(GEMAESbase.RTree,SETLITTT,SetLittleThresTree);
      ResourceGetAddr(GEMAESbase.RTree,HALTEDT,HaltedTree);
      ResourceGetAddr(GEMAESbase.RTree,INPUTERR,InputErrTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,NOTSAVED,NotSavedTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,ORBINFO,OrbInfoTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,SYSTIME,TimeDateTreeAddr);
      ResourceGetAddr(GEMAESbase.RTree,TLOG,LogTreeAddr);
      MenuBar(MenuTreeAddr,1);
      GrafMouse(GEMAESbase.Arrow,DummyAddress);
      InitBoolean:=TRUE
    END
  END;
  LoggingOn:=FALSE;
  OutOnScreen:=TRUE;
  GraphicsOut:=TRUE;
  RETURN InitBoolean
END Initialise;



PROCEDURE ToDesktop;
BEGIN
  IF ~OutOnScreen THEN
    CloseOutToPRN
  END;
  IF ScanInfoIsOpen THEN
    Message[0]:=22;
    Message[3]:=ScanInfoHandle;
    MessageScanInfo
  END;
  IF SatelInfoIsOpen THEN
    Message[0]:=22;
    Message[3]:=SatelInfoHandle;
    MessageSatelInfo
  END;
  IF StatInfoIsOpen THEN
    Message[0]:=22;
    Message[3]:=StatInfoHandle;
    MessageStatInfo
  END;
  IF OutWindowIsOpen THEN
    Message[0]:=22;
    Message[3]:=OutWindowHandle;
    MessageOutWindow
  END;
  IF LoggingOn THEN
    CloseOutToFile
  END;
  MenuBar(MenuTreeAddr,0);
  ResourceFree();
  CloseVirtualWorkstation(WorkstationHandle);
  ApplExit()
END ToDesktop;



PROCEDURE MenuInput;

  PROCEDURE DeskSelected;
  BEGIN
    IF Message[4]=ABOUT THEN
      HandleForm(OrbInfoTreeAddr,0)
    END
  END DeskSelected;

  PROCEDURE FileSelected;
  VAR Cont: BOOLEAN;
  BEGIN
    CASE Message[4] OF
      LOADSA:       LoadSatellite               |
      SAVESA:       SaveSatellite               |
      NEWSA:        NewSatellite                |
      CHANGESA:     ChangeSatellite             |
      DELETESA:     DeleteSatellite             |
      CHANGEST:     ChangeStation               |
      QUIT:         Cont:=TRUE;
                    IF SatelliteNotSaved THEN
                      SatNotSavedDialogue(Cont)
                    END;
                    IF Cont THEN
                      StopProgram:=TRUE;
                    END                         |
    ELSE
      Dummy:=FormAlert(1,'[3][Unknown menu item selected!][Abort]');
      StopProgram:=TRUE
    END
  END FileSelected;

  PROCEDURE ComputeSelected;
  BEGIN
    CASE Message[4] OF
      SETSCAN:  GetScanParams   |
      TRACK:    Track           |
      START:    Scan            |
    ELSE
      Dummy:=FormAlert(1,'[3][Unknown menu item selected!][Abort]');
      StopProgram:=TRUE
    END
  END ComputeSelected;

  PROCEDURE OutputSelected;
  VAR Ok: BOOLEAN;
  BEGIN
    CASE Message[4] OF
      SCREEN:   IF ~OutOnScreen THEN
                  MenuItemCheck(MenuTreeAddr,PRINTER,0);
                  CloseOutToPRN
                END;
                OutOnScreen:=TRUE;
                GraphicsOut:=FALSE;
                MenuItemEnable(MenuTreeAddr,ELOG,1);
                MenuItemCheck(MenuTreeAddr,GRAPHICS,0);
                MenuItemCheck(MenuTreeAddr,SCREEN,1)      |
      GRAPHICS: IF ~OutOnScreen THEN
                  MenuItemCheck(MenuTreeAddr,PRINTER,0);
                  CloseOutToPRN
                END;
                OutOnScreen:=TRUE;
                GraphicsOut:=TRUE;
                MenuItemEnable(MenuTreeAddr,ELOG,1);
                MenuItemCheck(MenuTreeAddr,SCREEN,0);
                MenuItemCheck(MenuTreeAddr,GRAPHICS,1)    |
      PRINTER:  IF OutOnScreen THEN
                  IF LoggingOn THEN
                    LoggingOn:=FALSE;
                    CloseOutToFile;
                  END;
                  MenuItemCheck(MenuTreeAddr,ELOG,0);
                  MenuItemEnable(MenuTreeAddr,ELOG,0);
                  OutOnScreen:=FALSE;
                  MenuItemCheck(MenuTreeAddr,SCREEN,0);
                  MenuItemCheck(MenuTreeAddr,GRAPHICS,0);
                  MenuItemCheck(MenuTreeAddr,PRINTER,1);
                  OutToPRN
                END                                       |
      ELOG:     IF LoggingOn THEN
                  CloseOutToFile;
                  LoggingOn:=FALSE;
                  MenuItemCheck(MenuTreeAddr,ELOG,0)
                ELSE
                  OutToFile(Ok);
                  IF Ok=TRUE THEN
                    LoggingOn:=TRUE;
                    MenuItemCheck(MenuTreeAddr,ELOG,1)
                  END
                END                                       |
      SHOWELEV: IF ShowLowElevation THEN
                  ShowLowElevation:=FALSE;
                  MenuItemCheck(MenuTreeAddr,SHOWELEV,0)
                ELSE
                  ShowLowElevation:=TRUE;
                  MenuItemCheck(MenuTreeAddr,SHOWELEV,1)
                END                                       |
      SETELEVA: GetElevationThres                         |
      SHOWLITT: IF ShowLittleChanges THEN
                  ShowLittleChanges:=FALSE;
                  MenuItemCheck(MenuTreeAddr,SHOWLITT,0)
                ELSE
                  ShowLittleChanges:=TRUE;
                  MenuItemCheck(MenuTreeAddr,SHOWLITT,1)
                END                                       |
      SETLITTL: GetLittleChangesThres                     |
    ELSE
      Dummy:=FormAlert(1,'[3][Unknown menu item selected!][Abort]');
      StopProgram:=TRUE
    END
  END OutputSelected;

  PROCEDURE InfoSelected;
  BEGIN
    CASE Message[4] OF
      SCANPARA: OpenScanInfo    |
      SALOADED: OpenSatelInfo   |
      STLOADED: OpenStatInfo    |
    ELSE
      Dummy:=FormAlert(1,'[3][Unknown menu item selected!][Abort]');
      StopProgram:=TRUE
    END
  END InfoSelected;

  PROCEDURE TimeDateSelected;
  BEGIN
    IF Message[4]=SETTIME THEN
      SetSystemTimeAndDate
    ELSE
      Dummy:=FormAlert(1,'[3][Unknown menu item selected!][Abort]');
      StopProgram:=TRUE
    END
  END TimeDateSelected;

BEGIN (* MenuInput *)
  CASE Message[3] OF
    DESK:     DeskSelected       |
    FILE:     FileSelected       |
    COMPUTE:  ComputeSelected    |
    OUTPUT:   OutputSelected     |
    INFO:     InfoSelected       |
    TIMEDATE: TimeDateSelected   |
  ELSE
    Dummy:=FormAlert(1,'[3][Unknown menu title selected!][Abort]');
    StopProgram:=TRUE
  END;
  MenuTitleNormal(MenuTreeAddr,Message[3],1)
END MenuInput;

PROCEDURE ThereIsMessage;
BEGIN
  CASE Message[0] OF
    GEMAESbase.MenuSelected:      MenuInput             |
    GEMAESbase.AccessoryOpen:     (* Do nothing *)      |
    GEMAESbase.AccessoryClose:    (* Do nothing *)      |
  ELSE
    IF ((Message[3]=ScanInfoHandle) & ScanInfoIsOpen) THEN
      MessageScanInfo
    ELSIF ((Message[3]=SatelInfoHandle) & SatelInfoIsOpen) THEN
      MessageSatelInfo
    ELSIF ((Message[3]=StatInfoHandle) & StatInfoIsOpen) THEN
      MessageStatInfo
    ELSIF ((Message[3]=OutWindowHandle) & OutWindowIsOpen) THEN
      MessageOutWindow
    ELSE
      Dummy:=FormAlert(1,'[1][Unknown message from event handler!][Ok]');
    END
  END
END ThereIsMessage;



PROCEDURE WaitForEvents;
BEGIN
  REPEAT
    EventMessage(ADR(Message));
    ThereIsMessage;
  UNTIL StopProgram
END WaitForEvents;



BEGIN  (* Main *)
  IF Initialise() THEN
    LoadStation;
    HandleForm(OrbInfoTreeAddr,0);
    LoadSatellite;
    WaitForEvents;
    ToDesktop
  ELSE
    CloseVirtualWorkstation(WorkstationHandle);
    ApplExit()
  END
END Orbit.
