IMPLEMENTATION MODULE OSatelInfo;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


FROM SYSTEM             IMPORT ADR;
FROM Strings            IMPORT String,
                               Assign,
                               Copy,
                               Concat,
                               Length;
FROM M2Conversions      IMPORT ConvertReal,
                               ConvertCardinal;

(********************** IMPORT from ORBIT specific libraries *)
FROM OFiler             IMPORT SatRec;
FROM OGEMGlobal         IMPORT MenuTreeAddr,
                               WorkstationHandle,
                               CharWidth,
                               CharHeight,
                               SALOADED,
                               Message;

(*********************) IMPORT GEMAESbase;
FROM AESWindows         IMPORT WindowGet,
                               WindowUpdate,
                               WindowCalc,
                               WindowCreate,
                               WindowClose,
                               WindowDelete,
                               WindowSet,
                               WindowOpen;
FROM AESMenus           IMPORT MenuItemEnable;
FROM AESGraphics        IMPORT GrafMouseKeyboardState,
                               GrafShrinkBox,
                               GrafGrowBox;
FROM AESForms           IMPORT FormAlert;

(*********************) IMPORT GEMVDIbase;
FROM VDIControls        IMPORT SetClipping;
FROM VDIOutputs         IMPORT FillRectangle,
                               GraphicText;
FROM VDIEscapes         IMPORT RemoveCursor,
                               DisplayCursor;


CONST InfoChars= 37;     (* Contents of window is 37 chars wide *)
      InfoLines= 12;     (* Contents of window is 12 lines      *)

VAR XDeskWork,YDeskWork,
    WDeskWork,HDeskWork,
    XTotal,YTotal,WTotal,HTotal,
    XWork,YWork,WWork,HWork,
    XOld,YOld,WOld,HOld,
    TextX,TextY,
    VSlidPos,HSlidPos,
    VSlidRelSize,HSlidRelSize,
    VisibleChars,VisibleLines,
    StartIndex,StartLine,
    WindowComponents:                    INTEGER;
    WindowName:                          String;
    PxyArray:                            GEMVDIbase.PxyArrayType;
    Contents:                            ARRAY[0..InfoLines] OF String;
    Fulled:                              BOOLEAN;



PROCEDURE ComputeHSlidPos;
BEGIN
  IF VisibleChars<InfoChars THEN
    HSlidPos:=((100*StartIndex) DIV (InfoChars-VisibleChars))*10
  ELSE
    HSlidPos:=1
  END
END ComputeHSlidPos;


PROCEDURE ComputeVSlidPos;
BEGIN
  IF VisibleLines<InfoLines THEN
    VSlidPos:=((100*StartLine) DIV (InfoLines-VisibleLines))*10
  ELSE
    VSlidPos:=1
  END
END ComputeVSlidPos;



PROCEDURE MakeContents;
VAR Str: String;
    X: CARDINAL;
BEGIN
  Assign(Contents[0],' Satellite Name:  ');
  Concat(Contents[0],SatRec.Naam,Contents[0]);
  Concat(Contents[0],'                  ',Contents[0]);
  Assign(Contents[1],' Year: (Ref. Epoch)              19');
  ConvertCardinal(SatRec.YJaar,1,Str);
  Copy(Str,Length(Str)-2,2,Str);
  Concat(Contents[1],Str,Contents[1]);
  Assign(Contents[2],' Epoch: (Decimal Day#)  ');
  ConvertReal(SatRec.YDag,13,-6,Str);
  Concat(Contents[2],Str,Contents[2]);
  Assign(Contents[3],' Orbit#:                ');
  ConvertCardinal(SatRec.YOmloop,13,Str);
  Concat(Contents[3],Str,Contents[3]);
  Assign(Contents[4],' Mean Anomaly:          ');
  ConvertReal(SatRec.YMeanAnom,13,-6,Str);
  Concat(Contents[4],Str,Contents[4]);
  Assign(Contents[5],' Mean Motion:           ');
  ConvertReal(SatRec.YMeanMotion,13,-6,Str);
  Concat(Contents[5],Str,Contents[5]);
  Assign(Contents[6],' Accel. Mean Motion:    ');
  ConvertReal(SatRec.YAccMeanM,13,-6,Str);
  Concat(Contents[6],Str,Contents[6]);
  Assign(Contents[7],' Inclination:           ');
  ConvertReal(SatRec.YInclinatie,13,-6,Str);
  Concat(Contents[7],Str,Contents[7]);
  Assign(Contents[8],' Eccentricity:          ');
  ConvertReal(SatRec.YEccentr,13,-6,Str);
  Concat(Contents[8],Str,Contents[8]);
  Assign(Contents[9],' Argument of Perigee:   ');
  ConvertReal(SatRec.YArgPer,13,-6,Str);
  Concat(Contents[9],Str,Contents[9]);
  Assign(Contents[10],' R.A.A.N.:              ');
  ConvertReal(SatRec.YRaan,13,-6,Str);
  Concat(Contents[10],Str,Contents[10]);
  Assign(Contents[11],' Frequency: (MHz)       ');
  ConvertReal(SatRec.Frekwentie,13,-6,Str);
  Concat(Contents[11],Str,Contents[11]);
  FOR X:=0 TO InfoChars DO
    Contents[InfoLines,X]:=' '
  END;
  FOR X:=0 TO InfoLines DO
    Contents[X,InfoChars]:=' '
  END
END MakeContents;



PROCEDURE UpdateSatelInfo;
VAR X,Y,W,H,
    MouseX,MouseY,MouseButton,
    MouseKeyboardState,
    StrNr:                            INTEGER;
    Str:                              String;
BEGIN
  MakeContents;
  WindowUpdate(GEMAESbase.BeginUpdate);
  GrafMouseKeyboardState(MouseX,MouseY,MouseButton,MouseKeyboardState);
  RemoveCursor(WorkstationHandle);
  WindowGet(SatelInfoHandle,GEMAESbase.FirstXYWH,X,Y,W,H);
  WHILE W#0 DO
    PxyArray[0]:=X;
    PxyArray[1]:=Y;
    PxyArray[2]:=X+W-1;
    PxyArray[3]:=Y+H-1;
    SetClipping(WorkstationHandle,1,PxyArray);
    IF WWork>(InfoChars*CharWidth) THEN
      PxyArray[0]:=XWork+InfoChars*CharWidth;
      FillRectangle(WorkstationHandle,PxyArray)
    END;
    IF HWork>(InfoLines*CharHeight) THEN
      PxyArray[0]:=X;
      PxyArray[1]:=YWork+InfoLines*CharHeight;
      FillRectangle(WorkstationHandle,PxyArray)
    END;
    TextX:=XWork;
    TextY:=YWork+CharHeight-3;
    FOR StrNr:=StartLine TO InfoLines DO
      Copy(Contents[StrNr],StartIndex,InfoChars-StartIndex+1,Str);
      GraphicText(WorkstationHandle,TextX,TextY,Str);
      TextY:=TextY+CharHeight
    END;
    WindowGet(SatelInfoHandle,GEMAESbase.NextXYWH,X,Y,W,H)
  END;
  DisplayCursor(WorkstationHandle,MouseX,MouseY);
  WindowUpdate(GEMAESbase.EndUpdate)
END UpdateSatelInfo;



PROCEDURE OpenSatelInfo;
BEGIN
  MenuItemEnable(MenuTreeAddr,SALOADED,0);
  WindowGet(0,GEMAESbase.WorkXYWH,XDeskWork,YDeskWork,WDeskWork,HDeskWork);
  WindowComponents:=0FFFH-GEMAESbase.InfoLine;
  SatelInfoHandle:=WindowCreate(WindowComponents,
                               XDeskWork,YDeskWork,WDeskWork,HDeskWork);
  WindowName:=" Satellite Info ";
  WindowSet(SatelInfoHandle,GEMAESbase.WindowName,
            INTEGER(LONGCARD(ADR(WindowName)) DIV 10000H),
            INTEGER(LONGCARD(ADR(WindowName)) MOD 10000H),0,0);
  VSlidRelSize:=1000;
  WindowSet(SatelInfoHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
  HSlidRelSize:=1000;
  WindowSet(SatelInfoHandle,GEMAESbase.HorizSliderSize,HSlidRelSize,0,0,0);
  VSlidPos:=1;
  WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
  HSlidPos:=1;
  WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
  XWork:=XDeskWork+5;
  YWork:=YDeskWork+25;
  WWork:=(InfoChars+1)*CharWidth;
  HWork:=InfoLines*CharHeight;
  Fulled:=FALSE;
  VisibleChars:=InfoChars+1;
  VisibleLines:=InfoLines;
  StartIndex:=0;
  StartLine:=0;
  WindowCalc(0,WindowComponents,XWork,YWork,WWork,HWork,
             XTotal,YTotal,WTotal,HTotal);
  GrafGrowBox(XTotal+(WTotal DIV 2),YTotal+(HTotal DIV 2),1,1,
              XTotal,YTotal,WTotal,HTotal);
  WindowOpen(SatelInfoHandle,XTotal,YTotal,WTotal,HTotal);
  XOld:=XTotal;
  YOld:=YTotal;
  WOld:=WTotal;
  HOld:=HTotal;
  UpdateSatelInfo;
  SatelInfoIsOpen:=TRUE
END OpenSatelInfo;



PROCEDURE SatelInfoClosed;
BEGIN
  WindowClose(SatelInfoHandle);
  GrafShrinkBox(XTotal+(WTotal DIV 2),YTotal+(HTotal DIV 2),1,1,
                XTotal,YTotal,WTotal,HTotal);
  WindowDelete(SatelInfoHandle);
  MenuItemEnable(MenuTreeAddr,SALOADED,1);
  SatelInfoIsOpen:=FALSE
END SatelInfoClosed;



PROCEDURE NewWindow(X,Y,W,H: INTEGER);
BEGIN
  IF ((W>50) & (H>50)) THEN
    XTotal:=X;
    YTotal:=Y;
    WTotal:=W;
    HTotal:=H;
    WindowSet(SatelInfoHandle,GEMAESbase.CurrXYWH,X,Y,W,H);
    WindowCalc(1,WindowComponents,X,Y,W,H,XWork,YWork,WWork,HWork);
    IF WWork>=InfoChars*CharWidth THEN
      HSlidPos:=1;
      HSlidRelSize:=1000;
      VisibleChars:=InfoChars;
      StartIndex:=0
    ELSE
      HSlidRelSize:=100*((10*WWork) DIV (InfoChars*CharWidth));
      VisibleChars:=WWork DIV CharWidth;
      IF VisibleChars>(InfoChars-StartIndex) THEN
        StartIndex:=InfoChars-VisibleChars
      END;
      ComputeHSlidPos
    END;
    IF HWork>=InfoLines*CharHeight THEN
      VSlidPos:=1;
      VSlidRelSize:=1000;
      VisibleLines:=InfoLines;
      StartLine:=0
    ELSE
      VSlidRelSize:=100*((10*HWork) DIV (InfoLines*CharHeight));
      VisibleLines:=HWork DIV CharHeight;
      IF VisibleLines>(InfoLines-StartLine) THEN
        StartLine:=InfoLines-VisibleLines
      END;
      ComputeVSlidPos
    END;
    WindowSet(SatelInfoHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
    WindowSet(SatelInfoHandle,GEMAESbase.HorizSliderSize,HSlidRelSize,0,0,0);
    WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
    WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
    UpdateSatelInfo;
    IF ((X#XDeskWork) OR (Y#YDeskWork) OR
        (W#WDeskWork) OR (H#HDeskWork))THEN
      XOld:=X;
      YOld:=Y;
      WOld:=W;
      HOld:=H;
      Fulled:=FALSE
    ELSE
      Fulled:=TRUE
    END
  END
END NewWindow;



PROCEDURE SatelInfoHorizSlided;
BEGIN
  HSlidPos:=Message[4];
  StartIndex:=((InfoChars-VisibleChars)*(HSlidPos DIV 10)) DIV 100;
  WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
  UpdateSatelInfo
END SatelInfoHorizSlided;



PROCEDURE SatelInfoVertSlided;
BEGIN
  VSlidPos:=Message[4];
  StartLine:=((InfoLines-VisibleLines)*(VSlidPos DIV 10)) DIV 100;
  WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
  UpdateSatelInfo
END SatelInfoVertSlided;



PROCEDURE SatelInfoArrowed;
VAR Dummy: INTEGER;

  PROCEDURE PageUp;
  BEGIN
    StartLine:=StartLine-VisibleLines+1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageUp;

  PROCEDURE PageDown;
  BEGIN
    StartLine:=StartLine+VisibleLines-1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageDown;

  PROCEDURE LineUp;
  BEGIN
    StartLine:=StartLine-1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineUp;

  PROCEDURE LineDown;
  BEGIN
    StartLine:=StartLine+1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineDown;

  PROCEDURE PageLeft;
  BEGIN
    StartIndex:=StartIndex-VisibleChars+1;
    IF StartIndex<0 THEN
      StartIndex:=0
    END;
    ComputeHSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END PageLeft;

  PROCEDURE PageRight;
  BEGIN
    StartIndex:=StartIndex+VisibleChars-1;
    IF (StartIndex+VisibleChars)>=InfoChars THEN
      StartIndex:=InfoChars-VisibleChars
    END;
    ComputeHSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END PageRight;

  PROCEDURE CharLeft;
  BEGIN
    StartIndex:=StartIndex-1;
    IF StartIndex<0 THEN
      StartIndex:=0
    END;
    ComputeHSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END CharLeft;

  PROCEDURE CharRight;
  BEGIN
    StartIndex:=StartIndex+1;
    IF (StartIndex+VisibleChars)>=InfoChars THEN
      StartIndex:=InfoChars-VisibleChars
    END;
    ComputeHSlidPos;
    WindowSet(SatelInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END CharRight;

BEGIN
  CASE Message[4] OF
    0: PageUp           |
    1: PageDown         |
    2: LineUp           |
    3: LineDown         |
    4: PageLeft         |
    5: PageRight        |
    6: CharLeft         |
    7: CharRight        |
  ELSE
    Dummy:=FormAlert(1,'[1][Unknown arrow function! |(Satellite info)][Ok]')
  END;
  UpdateSatelInfo
END SatelInfoArrowed;



PROCEDURE SatelInfoFulled;
BEGIN
  IF Fulled THEN
    GrafShrinkBox(XOld,YOld,WOld,HOld,
                  XDeskWork,YDeskWork,WDeskWork,HDeskWork);
    NewWindow(XOld,YOld,WOld,HOld)
  ELSE
    GrafGrowBox(XOld,YOld,WOld,HOld,
                XDeskWork,YDeskWork,WDeskWork,HDeskWork);
    NewWindow(XDeskWork,YDeskWork,WDeskWork,HDeskWork)
  END
END SatelInfoFulled;



PROCEDURE MessageSatelInfo;
VAR Dummy: INTEGER;
BEGIN
  CASE Message[0] OF
    GEMAESbase.WindowRedraw:      UpdateSatelInfo                          |
    GEMAESbase.WindowTopped:      WindowSet(SatelInfoHandle,GEMAESbase.Top,
                                            SatelInfoHandle,0,0,0)         |
    GEMAESbase.WindowClosed:      SatelInfoClosed                          |
    GEMAESbase.WindowFulled:      SatelInfoFulled                          |
    GEMAESbase.WindowArrowed:     SatelInfoArrowed                         |
    GEMAESbase.WindowHorizSlided: SatelInfoHorizSlided                     |
    GEMAESbase.WindowVertSlided:  SatelInfoVertSlided                      |
    GEMAESbase.WindowSized:       NewWindow(Message[4],Message[5],
                                            Message[6],Message[7])         |
    GEMAESbase.WindowMoved:       NewWindow(Message[4],Message[5],
                                            Message[6],Message[7])         |
  ELSE
    (* Do Nothing *)
  END
END MessageSatelInfo;


BEGIN
  SatelInfoIsOpen:=FALSE
END OSatelInfo.
