IMPLEMENTATION MODULE OStatInfo;

(* By: Harrie F.A. de Leeuw *)
(* Part of 'Orbit' program  *)


FROM SYSTEM             IMPORT ADR;
FROM Strings            IMPORT String,
                               Assign,
                               Copy,
                               Concat,
                               Length;
FROM M2Conversions      IMPORT ConvertReal,
                               ConvertCardinal,
                               ConvertInteger;

(********************** IMPORT from ORBIT specific libraries *)
FROM OFiler             IMPORT Station;
FROM OGEMGlobal         IMPORT MenuTreeAddr,
                               WorkstationHandle,
                               CharWidth,
                               CharHeight,
                               STLOADED,
                               Message;

(*********************) IMPORT GEMAESbase;
FROM AESWindows         IMPORT WindowGet,
                               WindowUpdate,
                               WindowCalc,
                               WindowCreate,
                               WindowClose,
                               WindowDelete,
                               WindowSet,
                               WindowOpen;
FROM AESMenus           IMPORT MenuItemEnable;
FROM AESGraphics        IMPORT GrafMouseKeyboardState,
                               GrafShrinkBox,
                               GrafGrowBox;
FROM AESForms           IMPORT FormAlert;

(*********************) IMPORT GEMVDIbase;
FROM VDIControls        IMPORT SetClipping;
FROM VDIOutputs         IMPORT FillRectangle,
                               GraphicText;
FROM VDIEscapes         IMPORT RemoveCursor,
                               DisplayCursor;


CONST InfoChars= 25;     (* Contents of window is 25 chars wide *)
      InfoLines= 5;      (* Contents of window is 5 lines       *)

VAR XDeskWork,YDeskWork,
    WDeskWork,HDeskWork,
    XTotal,YTotal,WTotal,HTotal,
    XWork,YWork,WWork,HWork,
    XOld,YOld,WOld,HOld,
    TextX,TextY,
    VSlidPos,HSlidPos,
    VSlidRelSize,HSlidRelSize,
    VisibleChars,VisibleLines,
    StartIndex,StartLine,
    WindowComponents:                    INTEGER;
    WindowName:                          String;
    PxyArray:                            GEMVDIbase.PxyArrayType;
    Contents:                            ARRAY[0..InfoLines] OF String;
    Fulled:                              BOOLEAN;



PROCEDURE ComputeHSlidPos;
BEGIN
  IF VisibleChars<InfoChars THEN
    HSlidPos:=((100*StartIndex) DIV (InfoChars-VisibleChars))*10
  ELSE
    HSlidPos:=1
  END
END ComputeHSlidPos;


PROCEDURE ComputeVSlidPos;
BEGIN
  IF VisibleLines<InfoLines THEN
    VSlidPos:=((100*StartLine) DIV (InfoLines-VisibleLines))*10
  ELSE
    VSlidPos:=1
  END
END ComputeVSlidPos;



PROCEDURE MakeContents;
VAR Str: String;
    X: CARDINAL;
BEGIN
  Assign(Contents[0],' Station Call:     ');
  Concat(Contents[0],Station.EigenNaam,Contents[0]);
  Concat(Contents[0],'      ',Contents[0]);
  Assign(Contents[1],' Latitude:  ');
  ConvertReal(Station.EigenNoord,13,-6,Str);
  Concat(Contents[1],Str,Contents[1]);
  Assign(Contents[2],' Longitude: ');
  ConvertReal(Station.EigenWest,13,-6,Str);
  Concat(Contents[2],Str,Contents[2]);
  Assign(Contents[3],' Height:    ');
  ConvertReal(Station.EigenHoogte,13,1,Str);
  Concat(Contents[3],Str,Contents[3]);
  Assign(Contents[4],' Local Time - GMT: ');
  ConvertInteger(Station.EigenTimeGMT,6,Str);
  Concat(Contents[4],Str,Contents[4]);
  FOR X:=0 TO InfoChars DO
    Contents[InfoLines,X]:=' '
  END;
  FOR X:=0 TO InfoLines DO
    Contents[X,InfoChars]:=' '
  END
END MakeContents;



PROCEDURE UpdateStatInfo;
VAR X,Y,W,H,
    MouseX,MouseY,MouseButton,
    MouseKeyboardState,
    StrNr:                            INTEGER;
    Str:                              String;
BEGIN
  MakeContents;
  WindowUpdate(GEMAESbase.BeginUpdate);
  GrafMouseKeyboardState(MouseX,MouseY,MouseButton,MouseKeyboardState);
  RemoveCursor(WorkstationHandle);
  WindowGet(StatInfoHandle,GEMAESbase.FirstXYWH,X,Y,W,H);
  WHILE W#0 DO
    PxyArray[0]:=X;
    PxyArray[1]:=Y;
    PxyArray[2]:=X+W-1;
    PxyArray[3]:=Y+H-1;
    SetClipping(WorkstationHandle,1,PxyArray);
    IF WWork>(InfoChars*CharWidth) THEN
      PxyArray[0]:=XWork+InfoChars*CharWidth;
      FillRectangle(WorkstationHandle,PxyArray)
    END;
    IF HWork>(InfoLines*CharHeight) THEN
      PxyArray[0]:=X;
      PxyArray[1]:=YWork+InfoLines*CharHeight;
      FillRectangle(WorkstationHandle,PxyArray)
    END;
    TextX:=XWork;
    TextY:=YWork+CharHeight-3;
    FOR StrNr:=StartLine TO InfoLines DO
      Copy(Contents[StrNr],StartIndex,InfoChars-StartIndex+1,Str);
      GraphicText(WorkstationHandle,TextX,TextY,Str);
      TextY:=TextY+CharHeight
    END;
    WindowGet(StatInfoHandle,GEMAESbase.NextXYWH,X,Y,W,H)
  END;
  DisplayCursor(WorkstationHandle,MouseX,MouseY);
  WindowUpdate(GEMAESbase.EndUpdate)
END UpdateStatInfo;



PROCEDURE OpenStatInfo;
BEGIN
  MenuItemEnable(MenuTreeAddr,STLOADED,0);
  WindowGet(0,GEMAESbase.WorkXYWH,XDeskWork,YDeskWork,WDeskWork,HDeskWork);
  WindowComponents:=0FFFH-GEMAESbase.InfoLine;
  StatInfoHandle:=WindowCreate(WindowComponents,
                               XDeskWork,YDeskWork,WDeskWork,HDeskWork);
  WindowName:=" Station Info ";
  WindowSet(StatInfoHandle,GEMAESbase.WindowName,
            INTEGER(LONGCARD(ADR(WindowName)) DIV 10000H),
            INTEGER(LONGCARD(ADR(WindowName)) MOD 10000H),0,0);
  VSlidRelSize:=1000;
  WindowSet(StatInfoHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
  HSlidRelSize:=1000;
  WindowSet(StatInfoHandle,GEMAESbase.HorizSliderSize,HSlidRelSize,0,0,0);
  VSlidPos:=1;
  WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
  HSlidPos:=1;
  WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
  XWork:=WDeskWork-InfoChars*CharWidth-30;
  YWork:=YDeskWork+25;
  WWork:=(InfoChars+1)*CharWidth;
  HWork:=InfoLines*CharHeight;
  Fulled:=FALSE;
  VisibleChars:=InfoChars+1;
  VisibleLines:=InfoLines;
  StartIndex:=0;
  StartLine:=0;
  WindowCalc(0,WindowComponents,XWork,YWork,WWork,HWork,
             XTotal,YTotal,WTotal,HTotal);
  GrafGrowBox(XTotal+(WTotal DIV 2),YTotal+(HTotal DIV 2),1,1,
              XTotal,YTotal,WTotal,HTotal);
  WindowOpen(StatInfoHandle,XTotal,YTotal,WTotal,HTotal);
  XOld:=XTotal;
  YOld:=YTotal;
  WOld:=WTotal;
  HOld:=HTotal;
  UpdateStatInfo;
  StatInfoIsOpen:=TRUE
END OpenStatInfo;



PROCEDURE StatInfoClosed;
BEGIN
  WindowClose(StatInfoHandle);
  GrafShrinkBox(XTotal+(WTotal DIV 2),YTotal+(HTotal DIV 2),1,1,
              XTotal,YTotal,WTotal,HTotal);
  WindowDelete(StatInfoHandle);
  MenuItemEnable(MenuTreeAddr,STLOADED,1);
  StatInfoIsOpen:=FALSE
END StatInfoClosed;



PROCEDURE NewWindow(X,Y,W,H: INTEGER);
BEGIN
  IF ((W>50) & (H>50)) THEN
    XTotal:=X;
    YTotal:=Y;
    WTotal:=W;
    HTotal:=H;
    WindowSet(StatInfoHandle,GEMAESbase.CurrXYWH,X,Y,W,H);
    WindowCalc(1,WindowComponents,X,Y,W,H,XWork,YWork,WWork,HWork);
    IF WWork>=InfoChars*CharWidth THEN
      HSlidPos:=1;
      HSlidRelSize:=1000;
      VisibleChars:=InfoChars;
      StartIndex:=0
    ELSE
      HSlidRelSize:=100*((10*WWork) DIV (InfoChars*CharWidth));
      VisibleChars:=WWork DIV CharWidth;
      IF VisibleChars>(InfoChars-StartIndex) THEN
        StartIndex:=InfoChars-VisibleChars
      END;
      ComputeHSlidPos
    END;
    IF HWork>=InfoLines*CharHeight THEN
      VSlidPos:=1;
      VSlidRelSize:=1000;
      VisibleLines:=InfoLines;
      StartLine:=0
    ELSE
      VSlidRelSize:=100*((10*HWork) DIV (InfoLines*CharHeight));
      VisibleLines:=HWork DIV CharHeight;
      IF VisibleLines>(InfoLines-StartLine) THEN
        StartLine:=InfoLines-VisibleLines
      END;
      ComputeVSlidPos
    END;
    WindowSet(StatInfoHandle,GEMAESbase.VertSliderSize,VSlidRelSize,0,0,0);
    WindowSet(StatInfoHandle,GEMAESbase.HorizSliderSize,HSlidRelSize,0,0,0);
    WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
    WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
    UpdateStatInfo;
    IF ((X#XDeskWork) OR (Y#YDeskWork) OR
        (W#WDeskWork) OR (H#HDeskWork))THEN
      XOld:=X;
      YOld:=Y;
      WOld:=W;
      HOld:=H;
      Fulled:=FALSE
    ELSE
      Fulled:=TRUE
    END
  END
END NewWindow;



PROCEDURE StatInfoHorizSlided;
BEGIN
  HSlidPos:=Message[4];
  StartIndex:=((InfoChars-VisibleChars)*(HSlidPos DIV 10)) DIV 100;
  WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0);
  UpdateStatInfo
END StatInfoHorizSlided;



PROCEDURE StatInfoVertSlided;
BEGIN
  VSlidPos:=Message[4];
  StartLine:=((InfoLines-VisibleLines)*(VSlidPos DIV 10)) DIV 100;
  WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0);
  UpdateStatInfo
END StatInfoVertSlided;



PROCEDURE StatInfoArrowed;
VAR Dummy: INTEGER;

  PROCEDURE PageUp;
  BEGIN
    StartLine:=StartLine-VisibleLines+1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageUp;

  PROCEDURE PageDown;
  BEGIN
    StartLine:=StartLine+VisibleLines-1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END PageDown;

  PROCEDURE LineUp;
  BEGIN
    StartLine:=StartLine-1;
    IF StartLine<0 THEN
      StartLine:=0
    END;
    ComputeVSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineUp;

  PROCEDURE LineDown;
  BEGIN
    StartLine:=StartLine+1;
    IF (StartLine+VisibleLines)>=InfoLines THEN
      StartLine:=InfoLines-VisibleLines
    END;
    ComputeVSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowVertSlide,VSlidPos,0,0,0)
  END LineDown;

  PROCEDURE PageLeft;
  BEGIN
    StartIndex:=StartIndex-VisibleChars+1;
    IF StartIndex<0 THEN
      StartIndex:=0
    END;
    ComputeHSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END PageLeft;

  PROCEDURE PageRight;
  BEGIN
    StartIndex:=StartIndex+VisibleChars-1;
    IF (StartIndex+VisibleChars)>=InfoChars THEN
      StartIndex:=InfoChars-VisibleChars
    END;
    ComputeHSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END PageRight;

  PROCEDURE CharLeft;
  BEGIN
    StartIndex:=StartIndex-1;
    IF StartIndex<0 THEN
      StartIndex:=0
    END;
    ComputeHSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END CharLeft;

  PROCEDURE CharRight;
  BEGIN
    StartIndex:=StartIndex+1;
    IF (StartIndex+VisibleChars)>=InfoChars THEN
      StartIndex:=InfoChars-VisibleChars
    END;
    ComputeHSlidPos;
    WindowSet(StatInfoHandle,GEMAESbase.WindowHorizSlide,HSlidPos,0,0,0)
  END CharRight;

BEGIN
  CASE Message[4] OF
    0: PageUp           |
    1: PageDown         |
    2: LineUp           |
    3: LineDown         |
    4: PageLeft         |
    5: PageRight        |
    6: CharLeft         |
    7: CharRight        |
  ELSE
    Dummy:=FormAlert(1,'[1][Unknown arrow function! |(Station info)][Ok]')
  END;
  UpdateStatInfo
END StatInfoArrowed;



PROCEDURE StatInfoFulled;
BEGIN
  IF Fulled THEN
    GrafShrinkBox(XOld,YOld,WOld,HOld,
                  XDeskWork,YDeskWork,WDeskWork,HDeskWork);
    NewWindow(XOld,YOld,WOld,HOld)
  ELSE
    GrafGrowBox(XOld,YOld,WOld,HOld,
                XDeskWork,YDeskWork,WDeskWork,HDeskWork);
    NewWindow(XDeskWork,YDeskWork,WDeskWork,HDeskWork)
  END
END StatInfoFulled;



PROCEDURE MessageStatInfo;
VAR Dummy: INTEGER;
BEGIN
  CASE Message[0] OF
    GEMAESbase.WindowRedraw:      UpdateStatInfo                          |
    GEMAESbase.WindowTopped:      WindowSet(StatInfoHandle,GEMAESbase.Top,
                                            StatInfoHandle,0,0,0)         |
    GEMAESbase.WindowClosed:      StatInfoClosed                          |
    GEMAESbase.WindowFulled:      StatInfoFulled                          |
    GEMAESbase.WindowArrowed:     StatInfoArrowed                         |
    GEMAESbase.WindowHorizSlided: StatInfoHorizSlided                     |
    GEMAESbase.WindowVertSlided:  StatInfoVertSlided                      |
    GEMAESbase.WindowSized:       NewWindow(Message[4],Message[5],
                                            Message[6],Message[7])        |
    GEMAESbase.WindowMoved:       NewWindow(Message[4],Message[5],
                                            Message[6],Message[7])        |
  ELSE
    (* Do Nothing *)
  END
END MessageStatInfo;


BEGIN
  StatInfoIsOpen:=FALSE
END OStatInfo.
